(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     11388,        340]
NotebookOptionsPosition[      9744,        279]
NotebookOutlinePosition[     10087,        294]
CellTagsIndexPosition[     10044,        291]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Computing integral via external function", "Title",
 CellChangeTimes->{{3.508481739028305*^9, 3.508481765715463*^9}}],

Cell["\<\

This notebook demonstrates how to compute  integral using external C++ \
function. \
\>", "Text",
 CellChangeTimes->{{3.489482325228853*^9, 3.489482362337516*^9}, {
   3.5084817831683645`*^9, 3.50848178393398*^9}, 3.5086621566834536`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"Needs", "[", "\"\<MathCode`\>\"", "]"}]}]], "Input",
 CellChangeTimes->{{3.485165972044566*^9, 3.4851659845129967`*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"MathCode C++ 1.4.4\"\>", "\[InvisibleSpace]", "\<\" for \"\>", 
   "\[InvisibleSpace]", "\<\"vc100\"\>", 
   "\[InvisibleSpace]", "\<\" loaded from \"\>", 
   "\[InvisibleSpace]", "\<\"C:\\\\b\\\\omath\\\\MathCode\"\>"}],
  SequenceForm[
  "MathCode C++ 1.4.4", " for ", "vc100", " loaded from ", 
   "C:\\b\\omath\\MathCode"],
  Editable->False]], "Print",
 CellChangeTimes->{3.4894823803996687`*^9, 3.5086621086059437`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"SetDirectory", "[", 
    RowBox[{
    "$MCRoot", "<>", "$PathnameSeparator", "<>", "\"\<Demos\>\"", "<>", 
     "$PathnameSeparator", "<>", "\"\<NIntegrateExternal\>\""}], "]"}], 
   ";"}]}]], "Input",
 CellChangeTimes->{{3.489481320154401*^9, 3.489481367965983*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myf", "[", 
    RowBox[{"Real", " ", "x_"}], "]"}], " ", "->", " ", "Real"}], " ", ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"x", "^", "2"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4851658554538007`*^9, 3.485165872000252*^9}, {
  3.485165908218075*^9, 3.485165915733507*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myf3", "[", 
    RowBox[{"Real", " ", "x_"}], "]"}], " ", "->", " ", "Real"}], " ", ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"x", "^", "3"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4851658554538007`*^9, 3.485165872000252*^9}, {
  3.485165908218075*^9, 3.485165915733507*^9}, {3.485169259087472*^9, 
  3.4851692617124043`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate", "[", 
  RowBox[{
   RowBox[{"myf", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "2", ",", "5"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.48516833926727*^9, 3.4851683654384747`*^9}}],

Cell[BoxData["39"], "Output",
 CellChangeTimes->{
  3.485168368079032*^9, 3.4851684117341647`*^9, 3.4851685180283184`*^9, 
   3.485169068389229*^9, 3.485169118466072*^9, 3.4851693410541234`*^9, 
   3.485169396615201*^9, 3.485169782042834*^9, 3.4851698475724063`*^9, 
   3.485169958678937*^9, 3.4851699973966956`*^9, 3.48948141473071*^9, 
   3.4894815187130885`*^9, {3.4894817786143484`*^9, 3.4894817908484883`*^9}, 
   3.4894818278321533`*^9, 3.489482380743412*^9, 3.5086621096684303`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"myf3", "[", "x", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "2", ",", "5"}], "}"}]}], "]"}], "//", "N"}]], "Input",
 CellChangeTimes->{{3.48516833926727*^9, 3.4851683654384747`*^9}, 
   3.4851692707434235`*^9, {3.4851697725118275`*^9, 3.48516977299619*^9}}],

Cell[BoxData["152.25`"], "Output",
 CellChangeTimes->{
  3.4851693411322465`*^9, 3.48516939664645*^9, 3.4851697820740833`*^9, 
   3.485169847588031*^9, 3.485169958710186*^9, 3.485169997427945*^9, 
   3.489481415777565*^9, 3.489481518759962*^9, {3.4894817786612225`*^9, 
   3.489481790910987*^9}, 3.489481827894652*^9, 3.489482380805911*^9, 
   3.5086621097309294`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"imyf", "[", 
     RowBox[{
      RowBox[{"Real", " ", "a_"}], ",", " ", 
      RowBox[{"Real", " ", "b_"}]}], "]"}], "\[Rule]", " ", "Real"}], " ", ":=", 
   RowBox[{"ExternalFunction", "[", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.485165894780919*^9, 3.4851659378735657`*^9}, {
  3.4851659911222024`*^9, 3.4851660118247976`*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"integrate1D", "[", 
      RowBox[{
       RowBox[{"String", " ", "fname_"}], ",", 
       RowBox[{"Real", " ", "a_"}], ",", 
       RowBox[{"Real", " ", "b_"}]}], "]"}], "\[Rule]", " ", "Real"}], ":=", 
    RowBox[{"ExternalFunction", "[", "]"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.485168475810649*^9, 3.4851685125128345`*^9}, 
   3.4851693923028116`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompilePackage", "[", 
  RowBox[{"NeedsExternalObjectModule", "\[Rule]", 
   RowBox[{"{", "\"\<imyfc\>\"", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.485166046792652*^9, 3.4851660494332094`*^9}, {
  3.4851661685239105`*^9, 3.485166185695346*^9}, {3.4851669972214456`*^9, 
  3.48516699879953*^9}}],

Cell[BoxData["\<\"Successful compilation to C++: 4 function(s)\"\>"], "Print",
 CellChangeTimes->{
  3.4851661283686886`*^9, 3.4851661886483955`*^9, 3.4851663166451187`*^9, {
   3.485166352441077*^9, 3.4851663669407063`*^9}, 3.4851679785108805`*^9, 
   3.4851684141559772`*^9, 3.4851685204501314`*^9, 3.485169070811042*^9, 
   3.485169120887885*^9, 3.4851693435853086`*^9, 3.485169399083888*^9, 
   3.4851697845271454`*^9, 3.485169850009844*^9, 3.4851699611476235`*^9, 
   3.485169999881007*^9, 3.4894814183868895`*^9, 3.4894815212755394`*^9, {
   3.489481781192424*^9, 3.489481793410939*^9}, 3.4894818304102287`*^9, 
   3.489482383305863*^9, 3.508662112184023*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"MakeBinary", "[", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"InstallCode", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Global\"\>", "\[InvisibleSpace]", "\<\" is installed.\"\>"}],
  SequenceForm["Global", " is installed."],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.489481533572178*^9, {3.489481785442342*^9, 3.489481797645233*^9}, 
   3.4894818346445227`*^9, 3.4894823877745275`*^9, 3.5086621164027195`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"imyf", "[", 
  RowBox[{"2.", ",", "5."}], "]"}]], "Input",
 CellChangeTimes->{{3.4851660767137613`*^9, 3.4851660895259333`*^9}, 
   3.485168328392548*^9}],

Cell[BoxData["39.00000000104774`"], "Output",
 CellChangeTimes->{
  3.485166135149765*^9, {3.4851683160803633`*^9, 3.4851683289862833`*^9}, 
   3.4851684173902698`*^9, 3.4851685239031677`*^9, 3.485169079545193*^9, 
   3.4851692386348705`*^9, 3.485169346803976*^9, 3.485169409817988*^9, 
   3.48516969351385*^9, 3.4851697879801817`*^9, 3.485169853478505*^9, 
   3.485169964460039*^9, 3.4851700033809175`*^9, 3.489481426652356*^9, 
   3.489481533634677*^9, {3.489481785457967*^9, 3.489481797676482*^9}, 
   3.489481834660147*^9, 3.489482387821401*^9, 3.508662116418344*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integrate1D", "[", 
  RowBox[{"\"\<myf\>\"", ",", "2.", ",", "5."}], "]"}]], "Input",
 CellChangeTimes->{{3.48516970991968*^9, 3.485169734419053*^9}}],

Cell[BoxData["39.00000000104774`"], "Output",
 CellChangeTimes->{{3.4851697298254204`*^9, 3.4851697351065354`*^9}, 
   3.4851697879958067`*^9, 3.4851698534941297`*^9, 3.485169964475663*^9, 
   3.485170003396542*^9, 3.489481426667981*^9, 3.4894815336659265`*^9, {
   3.489481785473592*^9, 3.4894817976921067`*^9}, 3.4894818346757717`*^9, 
   3.4894823878370256`*^9, 3.5086621164339685`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integrate1D", "[", 
  RowBox[{"\"\<myf3\>\"", ",", "2.", ",", "5."}], "]"}]], "Input",
 CellChangeTimes->{3.4851697693556585`*^9}],

Cell[BoxData["152.25000000275048`"], "Output",
 CellChangeTimes->{
  3.4851697880114307`*^9, 3.485169853509754*^9, 3.485169964491288*^9, 
   3.4851700034121666`*^9, 3.489481426683605*^9, 3.489481533681551*^9, {
   3.489481785489216*^9, 3.4894817977077312`*^9}, 3.489481834707021*^9, 
   3.489482387868275*^9, 3.5086621164339685`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integrate1D", "[", 
  RowBox[{"\"\<nonsense\>\"", ",", "2.", ",", "5."}], "]"}]], "Input",
 CellChangeTimes->{{3.485169831557191*^9, 3.4851698445099845`*^9}}],

Cell[BoxData[
 RowBox[{"-", "9999.`"}]], "Output",
 CellChangeTimes->{
  3.4851698585096264`*^9, {3.4851699744285336`*^9, 3.485170003427791*^9}, 
   3.48948142669923*^9, 3.4894815337128005`*^9, {3.489481785504841*^9, 
   3.4894817977389812`*^9}, 3.4894818347226458`*^9, 3.4894823878838997`*^9, 
   3.5086621164495935`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"UninstallCode", "[", "]"}], ";"}]], "Input"],

Cell["CleanMathCodeFiles[Confirm -> False, CleanAllBut -> {}];", "Input",
 CellChangeTimes->{{3.485169335460517*^9, 3.4851693389760513`*^9}, {
  3.4894813918873987`*^9, 3.48948139987162*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "[", "\"\<imyfc.obj\>\"", "]"}], ";", 
  RowBox[{"DeleteFile", "[", "\"\<imyfc.o\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.4851661086504436`*^9, 3.485166119853282*^9}}]
}, Open  ]]
},
WindowSize->{616, 750},
WindowMargins->{{334, Automatic}, {Automatic, 24}},
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (April 20, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 123, 1, 137, "Title"],
Cell[716, 26, 249, 6, 47, "Text"],
Cell[CellGroupData[{
Cell[990, 36, 178, 3, 52, "Input"],
Cell[1171, 41, 474, 10, 23, "Print"]
}, Open  ]],
Cell[1660, 54, 337, 8, 72, "Input"],
Cell[2000, 64, 379, 11, 31, "Input"],
Cell[2382, 77, 431, 12, 31, "Input"],
Cell[CellGroupData[{
Cell[2838, 93, 242, 6, 31, "Input"],
Cell[3083, 101, 489, 7, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3609, 113, 346, 8, 31, "Input"],
Cell[3958, 123, 368, 6, 30, "Output"]
}, Open  ]],
Cell[4341, 132, 397, 10, 31, "Input"],
Cell[4741, 144, 460, 12, 52, "Input"],
Cell[CellGroupData[{
Cell[5226, 160, 325, 6, 31, "Input"],
Cell[5554, 168, 665, 9, 23, "Print"]
}, Open  ]],
Cell[6234, 180, 58, 1, 31, "Input"],
Cell[CellGroupData[{
Cell[6317, 185, 77, 2, 31, "Input"],
Cell[6397, 189, 351, 7, 23, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6785, 201, 178, 4, 31, "Input"],
Cell[6966, 207, 571, 8, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7574, 220, 174, 3, 31, "Input"],
Cell[7751, 225, 389, 5, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8177, 235, 154, 3, 31, "Input"],
Cell[8334, 240, 333, 5, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8704, 250, 182, 3, 31, "Input"],
Cell[8889, 255, 322, 6, 30, "Output"]
}, Open  ]],
Cell[9226, 264, 79, 2, 31, "Input"],
Cell[9308, 268, 191, 2, 31, "Input"],
Cell[9502, 272, 226, 4, 31, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
