(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     38483,       1278]
NotebookOptionsPosition[     31354,       1074]
NotebookOutlinePosition[     31818,       1092]
CellTagsIndexPosition[     31775,       1089]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["MathCode setup test, v3 (2010-08-17)", "Title",
 CellChangeTimes->{{3.4746954908109636`*^9, 3.4746955018738885`*^9}, {
  3.491069232389038*^9, 3.4910692405542803`*^9}}],

Cell[TextData[{
 "\n",
 StyleBox["Always start MathCode  from Start Menu.",
  FontWeight->"Bold"],
 "\nThis Mathematica notebook tests the contents of Licensing directory and \
your MathCode setup in order to detect configuration errors. These errors \
occur if something unusual happens with your installation of your operating \
system, Mathematica, C++ or F90 compiler(s), MathCode or other software. \
Please read Release Notes, they explain most of these issues. \nPlease \
execute whole notebook, ",
 StyleBox["SAVE",
  FontWeight->"Bold"],
 " this notebook and send ",
 StyleBox["as attachment",
  FontWeight->"Bold"],
 " to support@mathcore.com.\nDo not cut-and-paste the text in this notebook \
into your e-mail text.\nIf you are ",
 StyleBox["concerned about privacy",
  FontWeight->"Bold"],
 " of information contained here you can remove any cells you do not like; \
you can also convert the notebook into a PDF file. \nWe use this notebook ",
 StyleBox["only",
  FontWeight->"Bold"],
 " to detect configuration errors in your installation of MathCode.  \n(C) \
2010, MathCore Engineerng AB. "
}], "Text",
 CellChangeTimes->{{3.4746954853888807`*^9, 3.4746954875295877`*^9}, {
   3.474695845979179*^9, 3.474695846447941*^9}, {3.5084819020730925`*^9, 
   3.5084819024480877`*^9}, 3.5086787356116657`*^9}],

Cell[CellGroupData[{

Cell["Before you start", "Subtitle"],

Cell[TextData[{
 StyleBox["Always start MathCode from Start Menu.",
  FontWeight->"Bold"],
 "\nFor MathCode C++ users: If you are trying to resolve a compiler \
configuration problem (BuildCode[] does not work) please try running in each  \
mode separately - with (default) \"mingw32\" first. If you use MicroSoft \
Visual C++ you should reinstall MathCode. So you get two notebooks."
}], "Text",
 CellChangeTimes->{{3.474695882323859*^9, 3.4746959078557625`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check version and license", "Subtitle"],

Cell[TextData[StyleBox["Always start MathCode from Start Menu.",
 FontWeight->"Bold"]], "Text"],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<MathCode`\>\"", "]"}]], "Input"],

Cell[TextData[{
 "Should normally tell ",
 StyleBox["MathCode (version) loaded.",
  FontWeight->"Bold"],
 " If it does (produce any errors) not then MathCode.m is missing in \
Mathematicas path. See  \
ToFileName[{$TopDirectory,\"AddOns\",\"Applications\"},\"MathCode.m\"] below. \
May be several Mathematica  installations on the same computer. MathCode \
Version number should be the same  as in setup executable.\n    "
}], "Text",
 CellChangeTimes->{3.49096108645632*^9, 3.491101715760006*^9}],

Cell[BoxData["$MCKey"], "Input"],

Cell["\<\
Tells user name specified during installation. It does not affect execution.  \
  \
\>", "Text"],

Cell[BoxData["$MCKeyData"], "Input"],

Cell["\<\
Tells organization name specified during installation. It does not affect \
execution.    \
\>", "Text"],

Cell[BoxData[
 RowBox[{"$MCLicenseID", "//", "FullForm"}]], "Input",
 CellChangeTimes->{3.491069456181282*^9}],

Cell["\<\
Tells license ID  specified during installation. It is compared with data \
encoded in license key delivered from MathCore by e-mail. \
\>", "Text"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"StringPosition", "[", 
     RowBox[{"$MCLicenseID", ",", "\"\<O\>\""}], "]"}], "\[NotEqual]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"Print", "[", "\"\<Bad license, letter O is present\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"StringMatchQ", "[", 
    RowBox[{"$MCLicenseID", ",", 
     RowBox[{
     "RegularExpression", "[", 
      "\"\<MC0\\\\d\\\\d\\\\d\\-\\\\d\\\\d\\\\d\\\\d\>\"", "]"}]}], "]"}], 
   ",", "Null", ",", 
   RowBox[{
   "Print", "[", "\"\<Bad license, does not match the pattern\>\"", "]"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"$F90License", "=", 
  RowBox[{
   RowBox[{"StringTake", "[", 
    RowBox[{"$MCLicenseID", ",", 
     RowBox[{"{", "6", "}"}]}], "]"}], "\[Equal]", "\"\<1\>\""}]}]], "Input",
 CellChangeTimes->{{3.491101805367508*^9, 3.49110181569*^9}}],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"$F90License", " ", "\[Equal]", " ", 
    RowBox[{"(", 
     RowBox[{
     "MathCode`$MathCodeDefaultLanguage", " ", "==", " ", 
      "\"\<Fortran90\>\""}], ")"}]}], ",", 
   RowBox[{"Print", "[", "\"\<OK\>\"", "]"}], ",", 
   RowBox[{"Print", "[", "\"\<Wrong target language\>\"", "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.491101805367508*^9, 3.491101820604538*^9}, {
   3.491101880132111*^9, 3.491101913195902*^9}, 3.5086780065162897`*^9}],

Cell["\<\
Whether this license is good for F90, and if this fact matches the license \
number. If not, you registered wrong variant of MathCode.\
\>", "Text",
 CellChangeTimes->{{3.491101805367508*^9, 3.491101812064458*^9}, {
  3.4911018536975517`*^9, 3.491101865176424*^9}}],

Cell[BoxData[
 RowBox[{"$MachineID", "//", "FullForm"}]], "Input"],

Cell["\<\
Tells license Machine  ID  of actual computer. It is compared  with data \
encoded in  license key delivered from MathCore by e-mail. \
\>", "Text"],

Cell[BoxData[
 RowBox[{"$MathCodeReleaseNumber", "//", "FullForm"}]], "Input"],

Cell["\<\
Tells the internal release number which MathCore increments with each new \
release.  \
\>", "Text"],

Cell[BoxData[
 RowBox[{"$MathCodeVersionNumber", "//", "FullForm"}]], "Input"],

Cell["\<\
Tells the internal version  number which MathCore increments with each new \
version.  \
\>", "Text"],

Cell[BoxData["$Version"], "Input"],

Cell[TextData[{
 "Tells the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "  version  number.  "
}], "Text",
 CellChangeTimes->{3.491100918133009*^9}],

Cell[BoxData[
 RowBox[{"$MCWindows", "=", 
  RowBox[{"$SystemID", "\[Equal]", "\"\<Windows\>\""}]}]], "Input",
 CellChangeTimes->{{3.491101025524544*^9, 3.491101075959792*^9}, 
   3.49110173950413*^9}],

Cell["Tells whether we assume Windows machine during this test.  ", "Text",
 CellChangeTimes->{
  3.491100918133009*^9, {3.491101939735282*^9, 3.491101957123822*^9}}],

Cell[BoxData["\[IndentingNewLine]"], "Input",
 CellChangeTimes->{3.491101925892899*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check where things are installed ", "Subtitle"],

Cell[BoxData[
 RowBox[{"Directory", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", 
  RowBox[{"If", "[", 
   RowBox[{"$MCWindows", ",", "\"\<C:\\\\\>\"", ",", "\"\</usr/local\>\""}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.4909612290202007`*^9, 3.4909612539629507`*^9}, {
  3.491101143042297*^9, 3.4911011807252693`*^9}, {3.491101218712884*^9, 
  3.491101235476783*^9}}],

Cell["\<\
Check what is installed in Program Files (makes sense on Windows only)\
\>", "Text",
 CellChangeTimes->{{3.491069554897461*^9, 3.491069605795867*^9}, {
  3.491101255060793*^9, 3.49110126295695*^9}}],

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"FileInformation", "[", "#", "]"}], ",", 
      RowBox[{"FileDate", "[", "#", "]"}]}], "}"}], "&"}], ",", 
   RowBox[{"FileNames", "[", "\"\<Pro*\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Length", "[", "%", "]"}], "\[Equal]", "1"}]], "Input"],

Cell["\<\
If it is false (on some Asian or French) Windows computers Progra~1 may point \
to different place. Reinstall Mathcode outside Progra* directories.  \
\>", "Text",
 CellChangeTimes->{{3.491069621446725*^9, 3.491069623051629*^9}}],

Cell[BoxData[
 RowBox[{"FileInformation", "[", "\"\<Progra~1\>\"", "]"}]], "Input"],

Cell["\<\
Check Mathematica installation
\
\>", "Text",
 CellChangeTimes->{{3.491101277053935*^9, 3.4911012845868*^9}}],

Cell[BoxData["$TopDirectory"], "Input"],

Cell[BoxData["$HomeDirectory"], "Input",
 CellChangeTimes->{{3.491101369975252*^9, 3.4911013784154243`*^9}}],

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{"ToFileName", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{"$MCWindows", ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "$TopDirectory", ",", "\"\<AddOns\>\"", ",", "\"\<Applications\>\""}], 
       "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
       "$HomeDirectory", ",", "\"\<.Mathematica\>\"", ",", 
        "\"\<Applications\>\""}], "}"}]}], "\[IndentingNewLine]", " ", "]"}], 
    ",", "\"\<MathCode.m\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.49110141124123*^9, 3.491101508460436*^9}}],

Cell[BoxData[
 RowBox[{"FileInformation", "[", "x", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"%", "\[Equal]", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{
   "Print", "[", 
    "\"\<Probably Mathcode was installed in a different place, or \
uninstalled, or not installec at all\>\"", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Import", "[", 
   RowBox[{"x", ",", "\"\<Text\>\""}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{3.49110153208922*^9}],

Cell["\<\
The values in this file should point to existing directories. This file makes \
possible to write Needs[\"MathCode`\"]. The details are checked later on.
    \
\>", "Text",
 CellChangeTimes->{{3.49110198660417*^9, 3.491102007955118*^9}}],

Cell[BoxData["$MCRoot"], "Input",
 CellChangeTimes->{{3.4911015445681334`*^9, 3.4911015493631887`*^9}}],

Cell[BoxData["MathCode`$MCRoot"], "Input"],

Cell[BoxData["MathCode`$MathCodeShortPath"], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", " ", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"%%", "\[NotEqual]", "%"}], ")"}], "||", 
     RowBox[{"(", 
      RowBox[{"%%%", "\[NotEqual]", "%"}], ")"}]}], ",", " ", 
    RowBox[{
    "Print", "[", 
     "\"\<MathCode context is not established, or MathCode is not correctly \
loaded.\>\"", "]"}]}], "]"}], "\[IndentingNewLine]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check licensing cache", "Subtitle"],

Cell[BoxData[
 RowBox[{"ToFileName", "[", 
  RowBox[{"{", 
   RowBox[{"MathCode`$MCRoot", ",", "\"\<Licensing\>\""}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", "$MCRoot", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"FileNames", "[", "]"}]], "Input"],

Cell[TextData[{
 "\nIf installation is correct, ",
 StyleBox["mathcode.lic",
  FontWeight->"Bold"],
 " must be here. Any ",
 StyleBox["*.mh",
  FontWeight->"Bold"],
 " files should not be here."
}], "Text",
 CellChangeTimes->{{3.4911020349476213`*^9, 3.4911020353236227`*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Import", "[", 
    RowBox[{"\"\<MathCode.lic\>\"", ",", "\"\<List\>\""}], "]"}], "//", 
   "FullForm"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"FileInformation", "[", "#", "]"}], ",", 
        RowBox[{"FileDate", "[", "#", "]"}]}], "}"}], "&"}], ",", 
     RowBox[{"FileNames", "[", "]"}]}], "]"}], "//", "TableForm"}]}]], "Input",
 CellChangeTimes->{{3.491069845542081*^9, 3.4910698554185*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"ToFileName", "[", 
    RowBox[{"{", 
     RowBox[{"MathCode`$MCRoot", ",", "\"\<Licensing\>\""}], "}"}], "]"}], 
   "]"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Directory", "[", "]"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Directory", "[", "]"}], "!=", 
    RowBox[{"ToFileName", "[", 
     RowBox[{
      RowBox[{"{", "MathCode`$MCRoot", "}"}], ",", "\"\<Licensing\>\""}], 
     "]"}]}], ",", 
   RowBox[{
   "Print", "[", 
    "\"\<The Licensing directory has not been created. MathCode has been \
installed without license key specified. See instructions how to put license \
key in this directory in an e-mail containing the license key\>\"", "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5086780891084538`*^9, 3.5086780928740067`*^9}}],

Cell[BoxData[
 RowBox[{"FileNames", "[", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"Map", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"FileInformation", "[", "#", "]"}], ",", 
       RowBox[{"FileDate", "[", "#", "]"}]}], "}"}], "&"}], ",", 
    RowBox[{"FileNames", "[", "]"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"FullForm", "[", 
      RowBox[{"Import", "[", 
       RowBox[{"#1", ",", "\"\<Text\>\""}], "]"}], "]"}], ")"}], "&"}], ",", 
   RowBox[{"FileNames", "[", "]"}]}], "]"}]], "Input"],

Cell[TextData[{
 "The key file usually named like ",
 StyleBox["234.mkey ",
  FontWeight->"Bold"],
 "or",
 " ",
 StyleBox["234.key",
  FontWeight->"Bold"],
 " should contain at least ",
 StyleBox["256",
  FontWeight->"Bold"],
 " bytes and no newlines in the middle. If not then it was wronlgy copied \
from e-mail attachment to this place. A message will be produced when you \
attempt to compile.\n       The index.m  should look like ",
 StyleBox["\"{\\\"6139-68937-93139\\\" -> \\\"234.key\\\"}\\n\"",
  FontWeight->"Bold"],
 ". If not, then ",
 StyleBox["$MachineID",
  FontWeight->"Bold"],
 " or  ",
 StyleBox["$MCLicenseID",
  FontWeight->"Bold"],
 " were wrong somewhere above. \n    "
}], "Text",
 CellChangeTimes->{{3.491069958485705*^9, 3.491069965134694*^9}}],

Cell[BoxData[
 RowBox[{"useKeyFile", "=", 
  RowBox[{
   RowBox[{"FileNames", "[", "\"\<*.*key\>\"", "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{3.491069948778255*^9}],

Cell["\<\
This should be the file you got from e - mail. Normally it is just one file. \
Any other  license key files should be better removed (except of shared \
installation case)
\
\>", "Text",
 CellChangeTimes->{{3.4911021355658283`*^9, 3.49110219440415*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check character codes ", "Subtitle"],

Cell["These tests return True on windows development computers. ", "Text"],

Cell[BoxData[
 StyleBox["$CharacterEncoding", "MR"]], "Input"],

Cell[BoxData[
 StyleBox["$SystemCharacterEncoding", "MR"]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["$CharacterEncoding", "MR"], 
  StyleBox["==", "MR"], "\"\<WindowsANSI\>\""}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox["$SystemCharacterEncoding", "MR"], 
  StyleBox["==", "MR"], "\"\<WindowsANSI\>\""}]], "Input"],

Cell[BoxData[
 RowBox[{"esca", "=", 
  "\"\<\\q\\w\\e(\\r)(\\t)\\y\\u\\i\\o\\p\\a\\s\\d(\\f)\\g\\h\\j\\k\\l\\z\\x\\\
c\\v(\\b)\\n\\m\\Q\n\\W\\E\\R\\T\\Y\\U\\I\\O\\P\\A\\S\\D\\F\\G\\H\\J\\K\\L\\Z\
\\X\\C\\V\\B\\N\\M\>\""}]], "Input",
 EmphasizeSyntaxErrors->True],

Cell[BoxData[
 RowBox[{"esca", "==", 
  "\"\<\\\\q\\\\w\\\\e(\\r)(\\t)\\\\y\\\\u\\\\i\\\\o\\\\p\\\\a\\\\s\\\\d(\\f)\
\\\\g\\\\h\\\\j\\\\k\\\\l\\\\z\\\\x\\\\c\\\\v(\.08)\\n\\\\m\\\\Q \\\\W\\\\E\\\
\\R\\\\T\\\\Y\\\\U\\\\I\\\\O\\\\P\\\\A\\\\S\\\\D\\\\F\\\\G\\\\H\\\\J\\\\K\\\\\
L\\\\Z\\\\X\\\\C\\\\V\\\\B\\\\N\\\\M\>\""}]], "Input"],

Cell[BoxData["\[IndentingNewLine]"], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check MathLink", "Subtitle"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"StringPosition", "[", 
       RowBox[{"$SystemID", ",", "\"\<Windows\>\""}], "]"}], "=!=", 
      RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"addTwoStr", "=", 
      RowBox[{
      "MathCode`$DevelopersKitsDirectoryFullPath", "<>", 
       "\"\<\\\\..\\\\PrebuiltExamples\\\\addtwo.exe\>\""}]}], ",", 
     RowBox[{"addTwoStr", "=", 
      RowBox[{
      "MathCode`$DevelopersKitsDirectoryFullPath", "<>", 
       "\"\</../PrebuiltExamples/addtwo\>\""}]}]}], "]"}], ";"}]}]], "Input"],

Cell["Missing file is evidence of non-standard paths to MathLink", "Text"],

Cell[BoxData[
 RowBox[{"FileInformation", "[", "addTwoStr", "]"}]], "Input"],

Cell["\<\
Failure to install this mathlink is evidence of firewall preventing process \
communication\
\>", "Text"],

Cell[BoxData[
 RowBox[{"linkAddTwo", "=", 
  RowBox[{"Install", "[", "addTwoStr", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"AddTwo", "[", 
  RowBox[{"2", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Uninstall", "[", "linkAddTwo", "]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check code generator", "Subtitle"],

Cell[BoxData[
 RowBox[{
  RowBox[{"tmplink", "=", 
   RowBox[{"Install", "[", 
    RowBox[{"$MCRoot", "<>", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"StringPosition", "[", 
         RowBox[{"$SystemID", ",", "\"\<Windows\>\""}], "]"}], "=!=", 
        RowBox[{"{", "}"}]}], ",", "\"\<\\\\System\\\\om.exe\>\"", ",", 
       "\"\</System/om\>\""}], "]"}]}], "]"}]}], 
  "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{"LinkPatterns", "[", "tmplink", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"LinkPatterns", "[", "tmplink", "]"}], "]"}], "\[Equal]", "9"}],
     ",", "OK", ",", 
    RowBox[{"Print", "[", "\"\<Missing patterns in OM.EXE\>\"", "]"}]}], 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"exid", "=", 
  RowBox[{"extractid", "[", 
   RowBox[{"ToFileName", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"MathCode`$MCRoot", ",", "\"\<Licensing\>\""}], "}"}], ",", 
     "useKeyFile"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.491102315981938*^9, 3.491102318299652*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"exid", " ", "\[Equal]", " ", "$MachineID"}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.491102322437832*^9, 3.491102336151267*^9}}],

Cell[TextData[{
 "If ",
 StyleBox["False",
  FontWeight->"Bold"],
 " - you have wrong license key file.\n   "
}], "Text",
 CellChangeTimes->{{3.4911023385452538`*^9, 3.4911023772939367`*^9}}],

Cell[BoxData[
 RowBox[{"licverify", "[", 
  RowBox[{"ToFileName", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"MathCode`$MCRoot", ",", "\"\<Licensing\>\""}], "}"}], ",", 
    "useKeyFile"}], "]"}], "]"}]], "Input"],

Cell[TextData[{
 "If not ",
 StyleBox["1",
  FontWeight->"Bold"],
 " your license expired"
}], "Text",
 CellChangeTimes->{{3.491102384460705*^9, 3.491102394940404*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"$DemosDir", "=", 
   RowBox[{"If", "[", 
    RowBox[{"$F90License", ",", "\"\<DemosF90\>\"", ",", "\"\<Demos\>\""}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.4910700833108873`*^9, 3.491070162928008*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{
   "$MCRoot", "<>", "\"\</\>\"", "<>", "$DemosDir", "<>", 
    "\"\</SimplestExample\>\""}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.491070167343381*^9, 3.491070192705284*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"Directory", "[", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
   "\"\<G.mci\>\"", ",", "\"\<model Global;  packages \\\"MathCode`\\\";\>\"",
     ",", "\"\<Text\>\""}], "]"}], "\n"}]], "Input"],

Cell["\<\

This must end with dead connection. Don't care.\
\>", "Text",
 CellChangeTimes->{{3.491102472055397*^9, 3.4911024836830997`*^9}}],

Cell[BoxData[
 RowBox[{"CompileInternal", "[", 
  RowBox[{"\"\<G.mci\>\"", ",", "\"\<G.mm\>\"", ",", "\"\<G.er\>\"", ",", 
   RowBox[{"If", "[", 
    RowBox[{
    "$F90License", ",", "\"\<-l Fortran90\>\"", ",", "\"\<-l C++\>\""}], 
    "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.491070229177425*^9, 3.4910702575961437`*^9}, 
   3.491070344163948*^9}],

Cell[BoxData[
 RowBox[{"Uninstall", "[", "tmplink", "]"}]], "Input"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.4910703867619123`*^9, 3.4910704300290813`*^9}, 
   3.491070484980933*^9}],

Cell[BoxData["\[IndentingNewLine]"], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check Windows language version", "Subtitle"],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", 
  RowBox[{"$MCRoot", "<>", "\"\</Demos\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Run", "[", "\"\<ver > x.txt\>\"", "]"}], 
  "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Import", "[", 
   RowBox[{"\"\<x.txt\>\"", ",", "\"\<Text\>\""}], "]"}], "//", 
  "FullForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}], 
  "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "Run", "[", 
   "\"\<START /WAIT REGEDIT /E \\\"x.txt\\\" \
\\\"HKEY_LOCAL_MACHINE\\\\Software\\\\Microsoft\\\\Windows \
NT\\\\CurrentVersion\\\\WOW\\\\boot.description\\\"\>\"", "]"}], 
  "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{"StringSplit", "[", 
  RowBox[{
   RowBox[{"Import", "[", 
    RowBox[{"\"\<x.txt\>\"", ",", 
     StyleBox["\"\<UnicodeText\>\"", "MR"]}], "]"}], ",", "\"\<\\n\>\""}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check compliers", "Subtitle"],

Cell["\<\
What you get here depends on MathCodeConfig.m, depends on whether you  done \
instructions from  Release Notes, and whether you have all necessary or \
optional  C++ compilers.
    \
\>", "Text"],

Cell[BoxData["$DefaultCompilers"], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"$CompilerDefinitions", "//", "FullForm"}]}]], "Input"],

Cell[BoxData["$SystemID"], "Input"],

Cell[BoxData["$System"], "Input"],

Cell[BoxData[
 RowBox[{"SetDirectory", "[", "$MCRoot", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Environment", "[", "\"\<COMSPEC\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Environment", "[", "\"\<MSDKBAT\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Environment", "[", "\"\<MCSTARTBAT\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{"$MCWindows", ",", 
   RowBox[{"Import", "[", 
    RowBox[{"\"\<StartMathCode.bat\>\"", ",", "\"\<Text\>\""}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.491102569637022*^9, 3.4911025850358458`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{"$MCWindows", ",", 
    RowBox[{"Import", "[", 
     RowBox[{"\"\<StartMathCode.bat\>\"", ",", "\"\<Text\>\""}], "]"}]}], 
   "]"}], "//", "FullForm"}]], "Input",
 CellChangeTimes->{{3.491102591495829*^9, 3.491102597707056*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Environment", "[", "\"\<MCSTARTBAT\>\"", "]"}], "//", 
  "FullForm"}]], "Input",
 CellChangeTimes->{{3.5086784767572613`*^9, 3.508678479632206*^9}}],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{"$MCWindows", ",", 
   RowBox[{"noquotes", "=", 
    RowBox[{"StringDrop", "[", 
     RowBox[{
      RowBox[{"StringDrop", "[", 
       RowBox[{
        RowBox[{"Environment", "[", "\"\<MCSTARTBAT\>\"", "]"}], ",", "1"}], 
       "]"}], ",", 
      RowBox[{"-", "2"}]}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.491102593824853*^9, 3.491102599821529*^9}, {
  3.508678553740158*^9, 3.508678554255773*^9}}],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{"$MCWindows", ",", 
   RowBox[{"Import", "[", 
    RowBox[{"noquotes", ",", "\"\<Text\>\""}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.491102608003469*^9, 3.491102614481372*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{"$MCWindows", ",", 
    RowBox[{"Import", "[", 
     RowBox[{"noquotes", ",", "\"\<Text\>\""}], "]"}]}], "]"}], "//", 
  "FullForm"}]], "Input",
 CellChangeTimes->{{3.491102609514134*^9, 3.491102618716165*^9}}],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{"$MCWindows", ",", 
   RowBox[{"noquotes", "=", 
    RowBox[{"StringDrop", "[", 
     RowBox[{
      RowBox[{"StringDrop", "[", 
       RowBox[{
        RowBox[{"Environment", "[", "\"\<MCSTARTBAT\>\"", "]"}], ",", "1"}], 
       "]"}], ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.491102593824853*^9, 3.491102599821529*^9}, {
  3.508678553740158*^9, 3.508678554255773*^9}, {3.508678627379369*^9, 
  3.508678627473117*^9}}],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{"$MCWindows", ",", 
   RowBox[{"Import", "[", 
    RowBox[{"noquotes", ",", "\"\<Text\>\""}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.491102608003469*^9, 3.491102614481372*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{"$MCWindows", ",", 
    RowBox[{"Import", "[", 
     RowBox[{"noquotes", ",", "\"\<Text\>\""}], "]"}]}], "]"}], "//", 
  "FullForm"}]], "Input",
 CellChangeTimes->{{3.491102609514134*^9, 3.491102618716165*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"DeleteFile", "::", "\"\<nffil\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"On", "[", 
  RowBox[{"DeleteFile", "::", "\"\<nffil\>\""}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Run", "[", "\"\<make -v > x.txt\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Import", "[", 
   RowBox[{"\"\<x.txt\>\"", ",", "\"\<Text\>\""}], "]"}], "//", 
  "FullForm"}]}], "Input"],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}]], "Input"],

Cell["\<\

Should report GNU Make here\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Run", "[", "\"\<set > x.txt\>\"", "]"}], ";", 
   RowBox[{
    RowBox[{"Import", "[", 
     RowBox[{"\"\<x.txt\>\"", ",", "\"\<Lines\>\""}], "]"}], "//", 
    "FullForm"}]}], "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}], 
  "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Run", "[", "\"\<assoc .h> x.txt\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Run", "[", "\"\<assoc  .cpp>> x.txt\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Run", "[", "\"\<assoc  .vcproj>> x.txt\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Import", "[", 
     RowBox[{"\"\<x.txt\>\"", ",", "\"\<Lines\>\""}], "]"}], "//", 
    "FullForm"}], "\[IndentingNewLine]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}], 
  "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Run", "[", "\"\<nmake /X- > x.txt\>\"", "]"}], ";", 
  RowBox[{
   RowBox[{"Import", "[", 
    RowBox[{"\"\<x.txt\>\"", ",", "\"\<Text\>\""}], "]"}], "//", 
   "FullForm"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Run", "[", "\"\<cl 2> x.txt\>\"", "]"}], ";", 
  RowBox[{
   RowBox[{"Import", "[", 
    RowBox[{"\"\<x.txt\>\"", ",", "\"\<Text\>\""}], "]"}], "//", 
   "FullForm"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Run", "[", "\"\<gcc -v 2> x.txt\>\"", "]"}], ";", 
  RowBox[{
   RowBox[{"Import", "[", 
    RowBox[{"\"\<x.txt\>\"", ",", "\"\<Text\>\""}], "]"}], "//", 
   "FullForm"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Run", "[", "\"\<g95 -v 2> x.txt\>\"", "]"}], ";", 
  RowBox[{
   RowBox[{"Import", "[", 
    RowBox[{"\"\<x.txt\>\"", ",", "\"\<Text\>\""}], "]"}], "//", 
   "FullForm"}]}]], "Input",
 CellChangeTimes->{{3.474695793462209*^9, 3.4746957948372445`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Run", "[", "\"\<type -p g95 > x.txt\>\"", "]"}], ";", 
  RowBox[{"FileInformation", "[", 
   RowBox[{"Import", "[", 
    RowBox[{"\"\<x.txt\>\"", ",", "\"\<Text\>\""}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.491102696873411*^9, 3.491102720608426*^9}, {
  3.491102845408905*^9, 3.491102891708213*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.491102918040525*^9, 3.491102921258794*^9}}],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", "\"\<x.txt\>\"", "]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Check normal BuildCode", "Subtitle"],

Cell["SetDirectory[$MCRoot<>\"/Demos\"];", "Input",
 CellChangeTimes->{3.491103171495284*^9}],

Cell["addtwo[Real a_,Real b_] -> Real := Module[{},a+b]", "Input"],

Cell[BoxData[
 RowBox[{"??", "addtwo"}]], "Input"],

Cell[TextData[{
 "Report about path without slashes is evidence of Japanses or Spanish \
Windows. Modify  ",
 StyleBox["MPREP=$(subst \\\\,\\,$(MPREP2))",
  FontWeight->"Bold"],
 "  to ",
 StyleBox["MPREP=$(MPREP2)",
  FontWeight->"Bold"],
 " in ",
 StyleBox["mingw32.mak",
  FontWeight->"Bold"],
 "\n"
}], "Text"],

Cell["BuildCode[]", "Input"],

Cell[BoxData[
 RowBox[{"FileNames", "[", "\"\<*\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Import", "[", 
   RowBox[{"\"\<Global.cmd\>\"", ",", "\"\<Lines\>\""}], "]"}], "//", 
  "FullForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Import", "[", 
   RowBox[{"\"\<Global.clog\>\"", ",", "\"\<Lines\>\""}], "]"}], "//", 
  "FullForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"link1", "=", 
   RowBox[{"Install", "[", "\"\<Globalml.exe\>\"", "]"}]}], 
  "\[IndentingNewLine]"}]], "Input"],

Cell[BoxData[
 RowBox[{"??", "addtwo"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"Uninstall", "[", "link1", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"??", "addtwo"}]}]], "Input"],

Cell["InstallCode[];", "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"??", "addtwo"}]}]], "Input"],

Cell["addtwo[2.,2.]", "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"UninstallCode", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
   "InputString", "[", 
    "\"\<In the following dialogs (if they appear) press: Cancel Cancel \
OK\>\"", "]"}], ";"}]}]], "Input",
 CellChangeTimes->{{3.491103259793797*^9, 3.491103267212302*^9}}],

Cell[BoxData[
 RowBox[{"FileNames", "[", "\"\<*\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"Run", "[", "\"\<./Globalml.exe\>\"", "]"}], "*)"}]}]], "Input",
 CellChangeTimes->{{3.491070998685007*^9, 3.491071002023736*^9}, {
  3.491103217796225*^9, 3.491103222083352*^9}, {3.49110343668147*^9, 
  3.491103440703703*^9}}],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{"Run", "[", 
   RowBox[{
    RowBox[{"FileNames", "[", "\"\<./Globalml.exe\>\"", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}], "]"}], "*)"}]], "Input",
 CellChangeTimes->{{3.4910710046339912`*^9, 3.491071007646987*^9}}],

Cell["CleanMathCodeFiles[Confirm -> False, CleanAllBut -> {}];", "Input"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1048, 831},
WindowMargins->{{93, Automatic}, {Automatic, 0}},
ShowSelection->True,
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.25, 1.25 Inherited],
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (April 20, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 174, 2, 103, "Title"],
Cell[767, 27, 1315, 28, 245, "Text"],
Cell[CellGroupData[{
Cell[2107, 59, 36, 0, 62, "Subtitle"],
Cell[2146, 61, 464, 8, 76, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2647, 74, 45, 0, 62, "Subtitle"],
Cell[2695, 76, 95, 1, 34, "Text"],
Cell[2793, 79, 74, 1, 35, "Input"],
Cell[2870, 82, 497, 10, 98, "Text"],
Cell[3370, 94, 32, 0, 35, "Input"],
Cell[3405, 96, 106, 3, 34, "Text"],
Cell[3514, 101, 36, 0, 35, "Input"],
Cell[3553, 103, 114, 3, 34, "Text"],
Cell[3670, 108, 110, 2, 35, "Input"],
Cell[3783, 112, 158, 3, 34, "Text"],
Cell[3944, 117, 295, 8, 35, "Input"],
Cell[4242, 127, 365, 11, 58, "Input"],
Cell[4610, 140, 259, 6, 35, "Input"],
Cell[4872, 148, 508, 12, 58, "Input"],
Cell[5383, 162, 275, 5, 34, "Text"],
Cell[5661, 169, 66, 1, 35, "Input"],
Cell[5730, 172, 158, 3, 34, "Text"],
Cell[5891, 177, 78, 1, 35, "Input"],
Cell[5972, 180, 110, 3, 34, "Text"],
Cell[6085, 185, 78, 1, 35, "Input"],
Cell[6166, 188, 111, 3, 34, "Text"],
Cell[6280, 193, 34, 0, 35, "Input"],
Cell[6317, 195, 157, 6, 34, "Text"],
Cell[6477, 203, 201, 4, 35, "Input"],
Cell[6681, 209, 166, 2, 34, "Text"],
Cell[6850, 213, 87, 1, 72, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6974, 219, 53, 0, 77, "Subtitle"],
Cell[7030, 221, 57, 1, 43, "Input"],
Cell[7090, 224, 341, 7, 43, "Input"],
Cell[7434, 233, 208, 4, 42, "Text"],
Cell[7645, 239, 282, 8, 43, "Input"],
Cell[7930, 249, 89, 2, 43, "Input"],
Cell[8022, 253, 239, 4, 42, "Text"],
Cell[8264, 259, 83, 1, 43, "Input"],
Cell[8350, 262, 119, 4, 68, "Text"],
Cell[8472, 268, 39, 0, 43, "Input"],
Cell[8514, 270, 108, 1, 43, "Input"],
Cell[8625, 273, 582, 15, 100, "Input"],
Cell[9210, 290, 68, 1, 43, "Input"],
Cell[9281, 293, 273, 8, 100, "Input"],
Cell[9557, 303, 174, 5, 72, "Input"],
Cell[9734, 310, 247, 5, 68, "Text"],
Cell[9984, 317, 103, 1, 43, "Input"],
Cell[10090, 320, 42, 0, 43, "Input"],
Cell[10135, 322, 53, 0, 43, "Input"],
Cell[10191, 324, 407, 12, 100, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10635, 341, 41, 0, 77, "Subtitle"],
Cell[10679, 343, 144, 4, 43, "Input"],
Cell[10826, 349, 71, 1, 43, "Input"],
Cell[10900, 352, 57, 1, 43, "Input"],
Cell[10960, 355, 276, 9, 70, "Text"],
Cell[11239, 366, 186, 5, 72, "Input"],
Cell[11428, 373, 413, 11, 72, "Input"],
Cell[11844, 386, 220, 6, 43, "Input"],
Cell[12067, 394, 88, 2, 43, "Input"],
Cell[12158, 398, 598, 15, 155, "Input"],
Cell[12759, 415, 57, 1, 43, "Input"],
Cell[12819, 418, 309, 9, 72, "Input"],
Cell[13131, 429, 270, 8, 43, "Input"],
Cell[13404, 439, 770, 24, 148, "Text"],
Cell[14177, 465, 202, 5, 43, "Input"],
Cell[14382, 472, 263, 6, 95, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14682, 483, 42, 0, 77, "Subtitle"],
Cell[14727, 485, 74, 0, 42, "Text"],
Cell[14804, 487, 62, 1, 43, "Input"],
Cell[14869, 490, 68, 1, 43, "Input"],
Cell[14940, 493, 123, 3, 43, "Input"],
Cell[15066, 498, 129, 3, 43, "Input"],
Cell[15198, 503, 262, 5, 72, "Input"],
Cell[15463, 510, 329, 5, 100, "Input"],
Cell[15795, 517, 45, 0, 72, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15877, 522, 34, 0, 77, "Subtitle"],
Cell[15914, 524, 628, 16, 127, "Input"],
Cell[16545, 542, 74, 0, 42, "Text"],
Cell[16622, 544, 76, 1, 43, "Input"],
Cell[16701, 547, 115, 3, 42, "Text"],
Cell[16819, 552, 100, 2, 43, "Input"],
Cell[16922, 556, 82, 2, 43, "Input"],
Cell[17007, 560, 71, 1, 43, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17115, 566, 40, 0, 77, "Subtitle"],
Cell[17158, 568, 435, 12, 100, "Input"],
Cell[17596, 582, 71, 1, 43, "Input"],
Cell[17670, 585, 319, 9, 72, "Input"],
Cell[17992, 596, 316, 8, 43, "Input"],
Cell[18311, 606, 183, 4, 72, "Input"],
Cell[18497, 612, 191, 6, 70, "Text"],
Cell[18691, 620, 219, 6, 43, "Input"],
Cell[18913, 628, 168, 6, 43, "Text"],
Cell[19084, 636, 268, 6, 72, "Input"],
Cell[19355, 644, 250, 6, 43, "Input"],
Cell[19608, 652, 93, 2, 72, "Input"],
Cell[19704, 656, 193, 5, 72, "Input"],
Cell[19900, 663, 140, 4, 68, "Text"],
Cell[20043, 669, 356, 8, 43, "Input"],
Cell[20402, 679, 68, 1, 43, "Input"],
Cell[20473, 682, 122, 2, 43, "Input"],
Cell[20598, 686, 45, 0, 72, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20680, 691, 50, 0, 77, "Subtitle"],
Cell[20733, 693, 108, 2, 43, "Input"],
Cell[20844, 697, 113, 3, 72, "Input"],
Cell[20960, 702, 140, 4, 43, "Input"],
Cell[21103, 708, 114, 3, 72, "Input"],
Cell[21220, 713, 260, 7, 127, "Input"],
Cell[21483, 722, 211, 6, 43, "Input"],
Cell[21697, 730, 75, 1, 43, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21809, 736, 35, 0, 77, "Subtitle"],
Cell[21847, 738, 205, 5, 95, "Text"],
Cell[22055, 745, 43, 0, 43, "Input"],
Cell[22101, 747, 112, 2, 72, "Input"],
Cell[22216, 751, 35, 0, 43, "Input"],
Cell[22254, 753, 33, 0, 43, "Input"],
Cell[22290, 755, 71, 1, 43, "Input"],
Cell[22364, 758, 78, 1, 43, "Input"],
Cell[22445, 761, 78, 1, 43, "Input"],
Cell[22526, 764, 81, 1, 43, "Input"],
Cell[22610, 767, 252, 6, 43, "Input"],
Cell[22865, 775, 285, 7, 43, "Input"],
Cell[23153, 784, 183, 4, 43, "Input"],
Cell[23339, 790, 458, 12, 43, "Input"],
Cell[23800, 804, 230, 5, 43, "Input"],
Cell[24033, 811, 267, 7, 43, "Input"],
Cell[24303, 820, 507, 13, 43, "Input"],
Cell[24813, 835, 230, 5, 43, "Input"],
Cell[25046, 842, 267, 7, 43, "Input"],
Cell[25316, 851, 323, 9, 100, "Input"],
Cell[25642, 862, 242, 7, 72, "Input"],
Cell[25887, 871, 75, 1, 43, "Input"],
Cell[25965, 874, 52, 3, 68, "Text"],
Cell[26020, 879, 256, 7, 72, "Input"],
Cell[26279, 888, 114, 3, 72, "Input"],
Cell[26396, 893, 546, 15, 182, "Input"],
Cell[26945, 910, 114, 3, 72, "Input"],
Cell[27062, 915, 220, 6, 43, "Input"],
Cell[27285, 923, 75, 1, 43, "Input"],
Cell[27363, 926, 214, 6, 43, "Input"],
Cell[27580, 934, 75, 1, 43, "Input"],
Cell[27658, 937, 218, 6, 43, "Input"],
Cell[27879, 945, 286, 7, 43, "Input"],
Cell[28168, 954, 344, 7, 43, "Input"],
Cell[28515, 963, 92, 1, 43, "Input"],
Cell[28610, 966, 75, 1, 43, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[28722, 972, 42, 0, 77, "Subtitle"],
Cell[28767, 974, 93, 1, 43, "Input"],
Cell[28863, 977, 66, 0, 43, "Input"],
Cell[28932, 979, 50, 1, 43, "Input"],
Cell[28985, 982, 314, 12, 97, "Text"],
Cell[29302, 996, 28, 0, 43, "Input"],
Cell[29333, 998, 70, 1, 43, "Input"],
Cell[29406, 1001, 146, 4, 43, "Input"],
Cell[29555, 1007, 147, 4, 43, "Input"],
Cell[29705, 1013, 146, 4, 72, "Input"],
Cell[29854, 1019, 50, 1, 43, "Input"],
Cell[29907, 1022, 102, 2, 72, "Input"],
Cell[30012, 1026, 86, 2, 72, "Input"],
Cell[30101, 1030, 31, 0, 43, "Input"],
Cell[30135, 1032, 86, 2, 72, "Input"],
Cell[30224, 1036, 30, 0, 43, "Input"],
Cell[30257, 1038, 79, 2, 43, "Input"],
Cell[30339, 1042, 268, 7, 72, "Input"],
Cell[30610, 1051, 70, 1, 43, "Input"],
Cell[30683, 1054, 302, 6, 72, "Input"],
Cell[30988, 1062, 262, 6, 43, "Input"],
Cell[31253, 1070, 73, 0, 43, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
