(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    100852,       2834]
NotebookOptionsPosition[     91518,       2512]
NotebookOutlinePosition[     92002,       2532]
CellTagsIndexPosition[     91959,       2529]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "System.nb file for implementation of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " system functions."
}], "Section",
 InitializationCell->True],

Cell["\<\
This file is a notebook file for implementation functions found in 
Mathematica, but which are missing outside Mathematica.

This file can easily be extended by adding additional function definitions.\
\>", "Text",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initialization for MathCode", "Section",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<MathCode`\>\"", "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"$MCRoot", "<>", "\"\</lib/stdpackages/src\>\""}], "]"}], 
  ";"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "The ",
 StyleBox["system",
  FontSlant->"Italic"],
 " package"
}], "Section",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["Preamble", "Subsection",
 InitializationCell->True],

Cell["Clear and remove all MathCode storages for these symbols", "Text",
 InitializationCell->True],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Map", "[", " ", 
   RowBox[{"clearDefinition", ",", " ", 
    RowBox[{"Map", "[", 
     RowBox[{"HoldSymbol", ",", " ", 
      RowBox[{"Names", "[", "\"\<system`*\>\"", "]"}]}], "]"}]}], "]"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"Remove", "[", "\"\<system`*\>\"", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Remove", "[", "\"\<system`*`*\>\"", "]"}], ";"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"HasInverse", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"$SystemID", "===", "\"\<Windows\>\""}], "||", 
      RowBox[{"$SystemID", "===", "\"\<Windows-x86-64\>\""}]}], ")"}], " ", "&&",
     " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
       "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
       ")"}], "!=", "\"\<mingw4\>\""}], ")"}]}]}], ";"}]], "Input",
 InitializationCell->True],

Cell["Package header", "Text",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"BeginPackage", "[", 
   RowBox[{"\"\<system`\>\"", ",", 
    RowBox[{"{", "MathCodeContexts", "}"}]}], "]"}], ";"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Public Exported global symbols", "Subsection",
 InitializationCell->True],

Cell["Begin[\"system`\"]; ", "Input",
 InitializationCell->True],

Cell["\<\
Introduce symbols that should be exported outside the package.\
\>", "Text",
 InitializationCell->True],

Cell["\<\
Note!! Important to add system` as context to force local creation of the 
symbols.\
\>", "Text",
 InitializationCell->True],

Cell["Off[General::shdw]", "Input",
 InitializationCell->True],

Cell["\<\
system`RotateLeft;
system`RotateRight;
system`Fourier;
system`InverseFourier;
system`Fourier3;
system`InverseFourier3;
If[HasInverse  ,
\tsystem`Inverse
];
system`Erf;
system`ErfOne;
system`ErfTwo;\
\>", "Input",
 InitializationCell->True],

Cell["On[General::shdw]", "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["End public section", "Subsubsection",
 InitializationCell->True],

Cell["End[]; ", "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Private package-global symbols", "Subsection",
 InitializationCell->True],

Cell["\<\
Introduce symbols that are not exported, but should be globally available 
within the package.\
\>", "Text",
 InitializationCell->True],

Cell["Begin[\"`Private`\"]; ", "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["End private-global section", "Subsubsection",
 InitializationCell->True],

Cell["End[]; ", "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Private implementation section", "Subsection",
 InitializationCell->True],

Cell["Begin[\"`Private`\"]; ", "Input",
 InitializationCell->True],

Cell["Define implementations of the functions and variables", "Text",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"HasInverse", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"$SystemID", "===", "\"\<Windows\>\""}], "||", 
      RowBox[{"$SystemID", "===", "\"\<Windows-x86-64\>\""}]}], ")"}], " ", "&&",
     " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
       "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
       ")"}], "!=", "\"\<mingw4\>\""}], ")"}]}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.509185391453315*^9, 3.50918539468769*^9}, {
  3.509185434640815*^9, 3.50918543687519*^9}}],

Cell[CellGroupData[{

Cell["RotateRight", "Subsubsection",
 InitializationCell->True],

Cell["\<\
RotateRight[ Integer[_]@ ar_]->Integer[_] :=  Module[{
   Integer@ m=Dimensions[ar][[1]]
  },Module[{
   Integer[m]@ res
  },
\tres[[2|m]]=ar[[1|m-1]];
    res[[1]]=ar[[m]];
    res
]];\
\>", "Input",
 InitializationCell->True],

Cell["RotRight example.", "Text",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["RotateRight[{1,2,3}]", "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "1", ",", "2"}], "}"}]], "Output",
 CellChangeTimes->{{3.509183873203315*^9, 3.50918388031269*^9}, {
   3.509184102078315*^9, 3.50918411031269*^9}, {3.50918430109394*^9, 
   3.50918430925019*^9}, 3.509184415109565*^9, 3.50918457131269*^9, 
   3.509184619797065*^9, 3.509184735797065*^9, 3.50918477525019*^9, {
   3.509185013172065*^9, 3.50918502146894*^9}, {3.509185065328315*^9, 
   3.509185073328315*^9}, 3.509185157140815*^9, 3.509185318047065*^9, 
   3.509185367922065*^9, 3.50918541090644*^9, 3.50918547062519*^9, {
   3.509185532297065*^9, 3.50918556843769*^9}, {3.509186140703315*^9, 
   3.509186148703315*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RotateLeft", "Subsubsection",
 InitializationCell->True],

Cell["\<\
RotateLeft [Integer[_]@ ar_]->Integer[_] :=  Module[{
   Integer@ m=Dimensions[ar][[1]]
  },Module[{
   Integer[m]@ res
  },
\tres[[1|m-1]]=ar[[2|m]];
    res[[m]]=ar[[1]];
    res
]];\
\>", "Input",
 InitializationCell->True],

Cell["RotateLeft example.", "Text",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["RotateLeft[{1,2,3,4}]=={2,3,4,1}", "Input",
 InitializationCell->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{{3.50918387325019*^9, 3.509183880328315*^9}, {
   3.509184102109565*^9, 3.509184110328315*^9}, {3.509184301109565*^9, 
   3.509184309265815*^9}, 3.509184415140815*^9, 3.509184571328315*^9, 
   3.509184619828315*^9, 3.509184735828315*^9, 3.509184775265815*^9, {
   3.509185013203315*^9, 3.50918502150019*^9}, {3.50918506534394*^9, 
   3.509185073359565*^9}, 3.509185157172065*^9, 3.50918531806269*^9, 
   3.50918536793769*^9, 3.509185410922065*^9, 3.509185470640815*^9, {
   3.50918553231269*^9, 3.509185568453315*^9}, {3.509186140734565*^9, 
   3.50918614871894*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Inverse", "Subsubsection",
 InitializationCell->True],

Cell[TextData[{
 "This implementation demonstrates how  to link the code generated by  \
MathCode with Fortran library files. The current directory contains a file  ",
 StyleBox["inverse.cpp  ",
  FontWeight->"Bold"],
 "which contain  definitions for function ",
 StyleBox[" inverse (in C++), dgetri (in Fortran) and dgetrf (in Fortran).",
  FontWeight->"Bold"],
 "  The function inverse calls ",
 StyleBox["dgetri",
  FontWeight->"Bold"],
 " and ",
 StyleBox["dgetrf",
  FontWeight->"Bold"],
 ". The functions ",
 StyleBox["dgetri",
  FontWeight->"Bold"],
 " and ",
 StyleBox["dgetrf",
  FontWeight->"Bold"],
 " are already compiled. The source code is given in",
 StyleBox[" dgetri.f ",
  FontWeight->"Bold"],
 "and",
 StyleBox[" dgetrf.f ",
  FontWeight->"Bold"],
 " They are part of LAPACK (Linear Algebra package) written in Fortran 77.  \
Four libraries are in this directory: ",
 StyleBox["blas.lib",
  FontWeight->"Bold"],
 " and ",
 StyleBox["lapack.lib",
  FontWeight->"Bold"],
 " (of LAPACK), ",
 StyleBox["dfor.lib",
  FontWeight->"Bold"],
 " and ",
 StyleBox["dfconsol.lib",
  FontWeight->"Bold"],
 " (Digital Visual Fortran (c) system libraries). "
}], "Text",
 InitializationCell->True],

Cell[TextData[{
 StyleBox["The functions we use should be declared with the keyword \
ExternalFunction  with optional prefix.\n If prefix is not specified for an \
external function,  then it is empty string. \n If prefix is not specified \
for a non-external  function, then it is  ", "Text"],
 StyleBox["packagename", "Text",
  FontWeight->"Bold",
  FontSlant->"Italic"],
 StyleBox["_T ", "Text",
  FontWeight->"Bold"],
 StyleBox[" (in this case", "Text"],
 StyleBox[" Objfiles_T", "Text",
  FontWeight->"Bold"],
 StyleBox[")", "Text"]
}], "Text",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{"HasInverse", ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Inverse", "[", 
         RowBox[{
          RowBox[{"Real", "[", 
           RowBox[{"_", ",", "_"}], "]"}], "@", "aff_"}], "]"}], "\[Rule]", 
        RowBox[{"Real", "[", 
         RowBox[{"_", ",", "_"}], "]"}]}], ":=", 
       RowBox[{"ExternalFunction", "[", 
        RowBox[{"aff", ",", 
         RowBox[{"ExternalPrefix", "\[Rule]", "\"\<System_T\>\""}]}], "]"}]}],
       ";"}]}], "]"}], ";"}], "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.50918493103144*^9, 3.50918494193769*^9}, {
  3.50918549903144*^9, 3.509185500390815*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Erf", "Subsubsection",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"ContinuedFractionApproxOne", "[", 
   RowBox[{"x_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "=", 
       RowBox[{"Range", "[", "n", "]"}]}], ",", "v", ",", "w"}], "}"}], ",", 
    RowBox[{
     RowBox[{"v", "=", 
      RowBox[{"Reverse", "[", "u", "]"}]}], ";", 
     RowBox[{"Fold", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", "#2"}], "-", "1"}], ")"}], "+", 
         RowBox[{
          SuperscriptBox["x", "2"], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "#2"], 
          FractionBox["#2", "#1"]}]}], "&"}], ",", "1", ",", "v"}], "]"}]}]}],
    "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ErfOne", "[", 
     RowBox[{"Real", "@", " ", "x_"}], "]"}], "\[Rule]", "Real"}], "=", 
   RowBox[{
    RowBox[{"2", 
     FractionBox[
      RowBox[{"Exp", "[", 
       RowBox[{"-", 
        SuperscriptBox["x", "2"]}], "]"}], 
      SqrtBox[
       RowBox[{"2", "\[Pi]"}]]], 
     FractionBox[
      RowBox[{
       SqrtBox["2"], "x"}], 
      RowBox[{"ContinuedFractionApproxOne", "[", 
       RowBox[{
        RowBox[{
         SqrtBox["2"], "x"}], ",", "20"}], "]"}]]}], "//", "N"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"ContinuedFractionApproxTwo", "[", 
   RowBox[{"x_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", "=", 
       RowBox[{"Range", "[", "n", "]"}]}], ",", "v", ",", "w"}], "}"}], ",", 
    RowBox[{
     RowBox[{"v", "=", 
      RowBox[{"Append", "[", 
       RowBox[{
        RowBox[{"Reverse", "[", "u", "]"}], ",", "1"}], "]"}]}], ";", 
     RowBox[{"w", "=", 
      RowBox[{
       RowBox[{"Fold", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"x", "+", 
           FractionBox["#2", "#1"]}], "&"}], ",", "1", ",", "v"}], "]"}], "-",
        "x"}]}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ErfTwo", "[", 
     RowBox[{"Real", "@", " ", "x_"}], "]"}], "\[Rule]", "Real"}], "=", 
   RowBox[{
    RowBox[{"1", "-", 
     RowBox[{"2", 
      FractionBox[
       RowBox[{"Exp", "[", 
        RowBox[{"-", 
         SuperscriptBox["x", "2"]}], "]"}], 
       SqrtBox[
        RowBox[{"2", "\[Pi]"}]]], 
      RowBox[{"ContinuedFractionApproxTwo", "[", 
       RowBox[{
        RowBox[{
         SqrtBox["2"], "x"}], ",", "20"}], "]"}]}]}], "//", "N"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Erf", "[", 
    RowBox[{"Real", "@", " ", "x_"}], "]"}], "\[Rule]", "Real"}], ":=", "\n", 
  "\t", 
  RowBox[{"Which", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "2."}], "\[LessEqual]", "x", "\[LessEqual]", "2."}], ",", 
    RowBox[{"ErfOne", "[", "x", "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"x", ">", "2."}], ",", 
    RowBox[{"ErfTwo", "[", "x", "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"x", "<", 
     RowBox[{"-", "2."}]}], ",", 
    RowBox[{"-", 
     RowBox[{"ErfTwo", "[", 
      RowBox[{"-", "x"}], "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fourier functions", "Subsubsection",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Fourier3", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Complex", "[", "_", "]"}], "u_"}], ",", 
      RowBox[{"Integer", " ", "af_"}], ",", 
      RowBox[{"Integer", " ", "b_"}]}], "]"}], "\[Rule]", 
    RowBox[{"Complex", "[", "_", "]"}]}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Integer", " ", "n"}], "=", 
        RowBox[{"Length", "[", "u", "]"}]}], ",", 
       RowBox[{
        RowBox[{"Complex", " ", "summ"}], "=", "0"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Complex", "[", "_", "]"}], " ", "v"}], "=", "u"}], ",", " ", 
       RowBox[{"Integer", " ", "r"}], ",", " ", 
       RowBox[{"Integer", " ", "s"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Do", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"summ", "=", "0"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Do", "[", " ", 
          RowBox[{
           RowBox[{"summ", "+=", 
            RowBox[{
             RowBox[{"u", "[", 
              RowBox[{"[", "r", "]"}], "]"}], " ", 
             RowBox[{"Exp", "[", 
              RowBox[{"2.", "Pi", " ", "\[ImaginaryI]", 
               RowBox[{"(", " ", 
                RowBox[{"b", "*", "1."}], ")"}], " ", 
               RowBox[{"(", 
                RowBox[{"r", "-", "1."}], ")"}], " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"s", "-", "1."}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"n", "*", "1."}], ")"}]}]}], "]"}]}]}], ",", 
           RowBox[{"{", 
            RowBox[{"r", ",", "1", ",", "n"}], "}"}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"v", "[", 
           RowBox[{"[", "s", "]"}], "]"}], "=", "summ"}], ";"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"s", ",", "1", ",", "n"}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"v", "*", 
       RowBox[{"(", 
        RowBox[{"1.", "/", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"n", " ", "*", "1."}], ")"}], "^", 
           RowBox[{"(", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1.", "-", "af"}], ")"}], "/", "2."}], ")"}], ")"}]}], 
          ")"}]}], ")"}]}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"InverseFourier3", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Complex", "[", "_", "]"}], "v_"}], ",", 
      RowBox[{"Integer", " ", "af_"}], ",", 
      RowBox[{"Integer", " ", "b_"}]}], "]"}], "\[Rule]", 
    RowBox[{"Complex", "[", "_", "]"}]}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Integer", " ", "n"}], "=", 
        RowBox[{"Length", "[", "v", "]"}]}], ",", 
       RowBox[{
        RowBox[{"Complex", " ", "summ"}], "=", "0"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Complex", "[", "_", "]"}], "u"}], "=", "v"}], ",", 
       RowBox[{"Integer", " ", "r"}], ",", " ", 
       RowBox[{"Integer", " ", "s"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Do", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"summ", "=", "0"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Do", "[", " ", 
          RowBox[{
           RowBox[{"summ", "+=", 
            RowBox[{
             RowBox[{"v", "[", 
              RowBox[{"[", "s", "]"}], "]"}], " ", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", " ", "2."}], "Pi", " ", "\[ImaginaryI]", 
               RowBox[{"(", " ", 
                RowBox[{"b", "*", "1."}], ")"}], 
               RowBox[{"(", 
                RowBox[{"r", "-", "1."}], ")"}], " ", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"s", "-", "1."}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"n", "*", "1."}], ")"}]}]}], "]"}]}]}], ",", 
           RowBox[{"{", 
            RowBox[{"s", ",", "1", ",", "n"}], "}"}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"u", "[", 
           RowBox[{"[", "r", "]"}], "]"}], "=", "summ"}], ";"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"r", ",", "1", ",", "n"}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"u", "*", 
       RowBox[{"(", 
        RowBox[{"1.", "/", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"n", " ", "*", "1."}], ")"}], "^", 
           RowBox[{"(", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1.", "+", "af"}], ")"}], "/", "2."}], ")"}], ")"}]}], 
          ")"}]}], ")"}]}]}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Fourier", "[", 
     RowBox[{
      RowBox[{"Complex", "[", "_", "]"}], " ", "aff_"}], "]"}], "\[Rule]", 
    RowBox[{"Complex", "[", "_", "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Fourier3", "[", 
      RowBox[{"aff", ",", "0", ",", "1"}], "]"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"InverseFourier", "[", 
     RowBox[{
      RowBox[{"Complex", "[", "_", "]"}], " ", "bff_"}], "]"}], "\[Rule]", 
    RowBox[{"Complex", "[", "_", "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"InverseFourier3", "[", 
      RowBox[{"bff", ",", "0", ",", "1"}], "]"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["End of private section", "Subsubsection",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["End[]", "Input",
 InitializationCell->True],

Cell[BoxData["\<\"system`Private`\"\>"], "Output",
 CellChangeTimes->{{3.50918387334394*^9, 3.50918388040644*^9}, {
   3.50918410221894*^9, 3.509184110390815*^9}, {3.50918430118769*^9, 
   3.509184309328315*^9}, 3.50918441521894*^9, 3.509184571422065*^9, 
   3.50918461990644*^9, 3.509184735953315*^9, 3.509184775390815*^9, {
   3.509185013390815*^9, 3.509185021640815*^9}, {3.509185065453315*^9, 
   3.509185073453315*^9}, 3.509185157297065*^9, 3.50918531818769*^9, 
   3.50918536806269*^9, 3.50918541103144*^9, 3.509185470765815*^9, {
   3.509185532390815*^9, 3.50918556853144*^9}, {3.509186140828315*^9, 
   3.50918614878144*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["End of package", "Subsection",
 InitializationCell->True],

Cell["EndPackage[]", "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compile the package  into C++ code", "Section",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["Generate C++ code", "Subsubsection",
 InitializationCell->True],

Cell["\<\
Specify the needed modules and libraries with absolute paths. This makes the 
options valid when this packages is refered from other packages compiled 
somewhere else.\
\>", "Text",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"$SystemID", "==", "\"\<Windows\>\""}], "||", 
       RowBox[{"$SystemID", "==", "\"\<Windows-x86-64\>\""}]}], ")"}], 
     "\[IndentingNewLine]", "&&", "\[IndentingNewLine]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
        "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
        ")"}], "==", "\"\<vc60\>\""}], ")"}]}], "\[IndentingNewLine]", ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", 
      RowBox[{"NeedsExternalObjectModule", " ", "->", " ", "\n", "    ", 
       RowBox[{"{", 
        RowBox[{
        "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
         "\"\<inverse\>\""}], "}"}]}], ",", "\n", "    ", 
      RowBox[{"NeedsExternalLibrary", " ", "->", "\n", "       ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
           "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
            "#"}], ")"}], "&"}], ")"}], "/@", "\n", "    \t  ", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Blas_DF60\>\"", ",", " ", "\"\<lapack_DF60\>\"", ",", " ", 
          "\"\<DFor\>\"", ",", " ", "\"\<DFConsol\>\"", ",", 
          "\"\<DFport\>\""}], "}"}]}]}], "\n", "         ", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"$SystemID", "==", "\"\<Windows\>\""}], "||", 
       RowBox[{"$SystemID", "==", "\"\<Windows-x86-64\>\""}]}], ")"}], 
     "\[IndentingNewLine]", "&&", "\[IndentingNewLine]", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
         "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
         ")"}], "==", "\"\<vc70\>\""}], "\[IndentingNewLine]", "||", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
         "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
         ")"}], "==", "\"\<vc71\>\""}]}], "\[IndentingNewLine]", ")"}]}], 
    "\[IndentingNewLine]", ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", 
      RowBox[{"NeedsExternalObjectModule", " ", "->", " ", "\n", "    ", 
       RowBox[{"{", 
        RowBox[{
        "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
         "\"\<inverse\>\""}], "}"}]}], ",", "\n", "    ", 
      RowBox[{"NeedsExternalLibrary", " ", "->", "\n", "       ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
           "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
            "#"}], ")"}], "&"}], ")"}], "/@", "\n", "    \t  ", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Blas_IFORT80\>\"", ",", " ", "\"\<lapack_IFORT80\>\"", ",", 
          "\"\<ifconsol\>\"", ",", "\"\<libifcore\>\"", ",", 
          "\"\<libifport\>\"", ",", "\"\<libirc\>\"", ",", "\"\<libm\>\"", 
          ",", "\"\<tmglib_ifort80\>\""}], "}"}]}]}], "\n", "         ", ",", 
      "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(", 
  RowBox[{"MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
  ")"}]], "Input"],

Cell[BoxData["\<\"vc100\"\>"], "Output",
 CellChangeTimes->{3.50918421159394*^9, 3.509184309359565*^9, 
  3.50918441525019*^9, 3.509184571453315*^9, 3.50918461993769*^9, 
  3.509184736015815*^9, 3.509184775422065*^9, 3.50918502168769*^9, 
  3.509185073484565*^9, 3.509185157328315*^9, 3.50918531825019*^9, 
  3.50918536812519*^9, 3.50918541109394*^9, 3.509185470828315*^9, 
  3.509185532422065*^9, 3.509185568547065*^9, 3.50918614881269*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"$SystemID", "==", "\"\<Windows\>\""}], "||", 
       RowBox[{"$SystemID", "==", "\"\<Windows-x86-64\>\""}]}], ")"}], 
     "\[IndentingNewLine]", "&&", "\[IndentingNewLine]", 
     RowBox[{"(", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
         "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
         ")"}], "==", "\"\<vc80\>\""}], "\[IndentingNewLine]", "||", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
         "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
         ")"}], "==", "\"\<vc81\>\""}], "\[IndentingNewLine]", "||", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
         "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
         ")"}], "==", "\"\<vc90\>\""}], "\[IndentingNewLine]", "||", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
         "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
         ")"}], "==", "\"\<vc100\>\""}]}], "\[IndentingNewLine]", ")"}]}], 
    "\[IndentingNewLine]", ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", 
      RowBox[{"NeedsExternalObjectModule", " ", "->", " ", "\n", "    ", 
       RowBox[{"{", 
        RowBox[{
        "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
         "\"\<inverse\>\""}], "}"}]}], ",", "\n", "    ", 
      RowBox[{"NeedsExternalLibrary", " ", "->", "\n", "       ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
           "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
            "#"}], ")"}], "&"}], ")"}], "/@", "\n", "    \t  ", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Blas_IFORT91\>\"", ",", " ", "\"\<lapack_IFORT91\>\"", ",", 
          "\"\<ifconsol\>\"", ",", "\"\<libifcoremt\>\"", ",", 
          "\"\<libifport\>\"", ",", "\"\<libirc\>\"", ",", "\"\<libmmt\>\""}],
          "}"}]}]}], "\n", "         ", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", 
     "\[IndentingNewLine]", " ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.50918400162519*^9, 3.509184013265815*^9}, {
   3.50918438659394*^9, 3.509184401140815*^9}, {3.50918465643769*^9, 
   3.50918465978144*^9}, 3.50918559853144*^9}],

Cell[BoxData["\<\"Successful compilation to C++: 10 function(s)\"\>"], "Print",
 CellChangeTimes->{{3.50918410459394*^9, 3.509184112890815*^9}, {
   3.50918430356269*^9, 3.50918431184394*^9}, 3.509184417609565*^9, 
   3.50918457381269*^9, 3.509184622297065*^9, 3.509184662078315*^9, 
   3.50918473618769*^9, 3.509184775578315*^9, {3.509185013578315*^9, 
   3.509185021984565*^9}, {3.50918506562519*^9, 3.509185073765815*^9}, 
   3.509185157515815*^9, 3.50918531840644*^9, 3.50918536828144*^9, 
   3.50918541125019*^9, 3.509185470984565*^9, {3.509185532578315*^9, 
   3.50918556884394*^9}, {3.509186143203315*^9, 3.509186151297065*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"$SystemID", "==", "\"\<Windows\>\""}], "||", 
       RowBox[{"$SystemID", "==", "\"\<Windows-x86-64\>\""}]}], ")"}], 
     "\[IndentingNewLine]", "&&", "\[IndentingNewLine]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
        "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
        ")"}], "==", "\"\<mingw32\>\""}], ")"}]}], "\[IndentingNewLine]", ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", 
      RowBox[{"NeedsExternalObjectModule", " ", "->", " ", "\n", "    ", 
       RowBox[{"{", 
        RowBox[{
        "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
         "\"\<inverse\>\""}], "}"}]}], ",", "\n", "    ", 
      RowBox[{"NeedsExternalLibrary", " ", "->", "\n", "       ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
           "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
            "#"}], ")"}], "&"}], ")"}], "/@", "\n", "    \t  ", 
        RowBox[{"{", 
         RowBox[{
         "\"\<lapack_MING77\>\"", ",", " ", "\"\<blas_MING77\>\"", ",", " ", 
          "\"\<libg2c_MING77\>\""}], "}"}]}]}], "\n", "         ", ",", "\n", 
      "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"$SystemID", "==", "\"\<Windows\>\""}], "||", 
       RowBox[{"$SystemID", "==", "\"\<Windows-x86-64\>\""}]}], ")"}], 
     "\[IndentingNewLine]", "&&", "\[IndentingNewLine]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
        "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
        ")"}], "==", "\"\<mingw4\>\""}], ")"}]}], "\[IndentingNewLine]", ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Disk2CygwinStyle", "[", "dosname_String", "]"}], ":=", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"StringTake", "[", 
        RowBox[{"dosname", ",", 
         RowBox[{"{", "2", "}"}]}], "]"}], "==", "\"\<:\>\""}], ",", 
      RowBox[{"\"\</cygdrive/\>\"", "<>", 
       RowBox[{"StringTake", "[", 
        RowBox[{"dosname", ",", 
         RowBox[{"{", "1", "}"}]}], "]"}], "<>", "\"\</\>\"", "<>", 
       RowBox[{"StringDrop", "[", 
        RowBox[{"dosname", ",", "2"}], "]"}]}], ",", "dosname"}], "]"}]}], 
   ";"}], "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"$SystemID", "==", "\"\<Windows\>\""}], "||", 
       RowBox[{"$SystemID", "==", "\"\<Windows-x86-64\>\""}]}], ")"}], "&&", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
        "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
        ")"}], "==", "\"\<cygwin\>\""}], ")"}]}], "\[IndentingNewLine]", ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", 
      RowBox[{"NeedsExternalObjectModule", " ", "->", " ", "\n", "    ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
         "Disk2CygwinStyle", "[", "MathCode`$MathCodeSystemlib", "]"}], "<>", 
         "$PathnameSeparator", "<>", "\"\<inverse\>\""}], "}"}]}], ",", "\n", 
      "    ", 
      RowBox[{"NeedsExternalLibrary", " ", "->", "\n", "       ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
            "Disk2CygwinStyle", "[", "MathCode`$MathCodeSystemlib", "]"}], "<>",
             "$PathnameSeparator", "<>", "#"}], ")"}], "&"}], ")"}], "/@", 
        "\n", "    \t  ", 
        RowBox[{"{", 
         RowBox[{
         "\"\<lapack_cygwin77\>\"", ",", " ", "\"\<blas_cygwin77\>\""}], 
         "}"}]}]}], "\n", "         ", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{3.50918453296894*^9, 3.509184598203315*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"$SystemID", "==", "\"\<Windows\>\""}], "||", 
        RowBox[{"$SystemID", "==", "\"\<Windows-x86-64\>\""}]}], ")"}], 
      "\[IndentingNewLine]", "&&", "\[IndentingNewLine]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
         "MathCode`$MathCodeDefaultLanguage", "/.", "$DefaultCompilers"}], 
         ")"}], "==", "\"\<mingw32\>\""}], ")"}]}], "\[IndentingNewLine]", 
     ",", 
     RowBox[{"CompilePackage", "[", 
      RowBox[{"\"\<system\>\"", "  ", ",", "\n", "\t", 
       RowBox[{"EvaluateFunctions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", "*)"}], 
  "\[IndentingNewLine]"}]], "Input",
 Evaluatable->False,
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$SystemID", "==", "\"\<Solaris\>\""}], ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", 
      RowBox[{"NeedsExternalObjectModule", " ", "->", " ", "\n", "    ", 
       RowBox[{"{", 
        RowBox[{
        "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
         "\"\<inverse\>\""}], "}"}]}], ",", "\n", "    ", 
      RowBox[{"NeedsExternalLibrary", " ", "->", "\n", "       ", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
             "MathCode`$MathCodeSystemlib", "<>", "$PathnameSeparator", "<>", 
              "#"}], ")"}], "&"}], ")"}], "/@", "\n", "    \t          ", 
          RowBox[{"{", " ", 
           RowBox[{"\"\<lapack\>\"", ",", "\"\<Blas\>\""}], "}"}]}], ",", 
         "\n", "    \t   ", 
         RowBox[{"{", 
          RowBox[{
          "\"\</opt/SUNWspro/SC4.0/lib/libF77.a\>\"", ",", " ", "\n", 
           "            ", "\"\</opt/SUNWspro/SC4.0/lib/libsunmath.a\>\""}], 
          "}"}]}], "\n", "         ", "\n", "         ", "]"}]}], ",", "\n", 
      "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$SystemID", "==", "\"\<UltraSPARC\>\""}], ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$SystemID", "==", "\"\<Linux\>\""}], ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$SystemID", "==", "\"\<Linux-x86-64\>\""}], ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$SystemID", "==", "\"\<MacOSX-x86\>\""}], ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$SystemID", "==", "\"\<MacOSX-x86-64\>\""}], ",", 
    RowBox[{"CompilePackage", "[", 
     RowBox[{"\"\<system\>\"", ",", "\n", "\t", 
      RowBox[{"EvaluateFunctions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"ErfOne", ",", "ErfTwo"}], "}"}]}]}], "  ", "]"}]}], "]"}], 
  "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["The generated C++ code", "Subsubsection",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["!!system.cc", "Input",
 InitializationCell->True],

Cell["\<\
#include \"system.h\"

#include \"system.icc\"

#include <math.h>
double system_TErf ( const double &x)
{
    return ( (-2. <= x && x <= 2.) ? (system_TErfOne (x)) : ( (x > 2.) ? (
        system_TErfTwo (x)) : ( (x < -2.) ? (-system_TErfTwo (-x)) :  ( -
        system_TErfTwo (-x) ) )));
}

lm_complexN system_TFourier ( const lm_complexN &aff)
{
    return system_TFourier3 (aff, 0, 1);
}

lm_complexN system_TFourier3 ( const lm_complexN &u, const int &af, const int \
&b
    )
{
    int n;
    lm_complex summ = 0;
    lm_complexN v(0);
    v = u;
    int r;
    int s;
    int r_D3725;
    int s_D3728;

    n = u.dimension(1);
    s_D3728 = 1;
    while (s_D3728 <= n)
    {
        summ = 0;
        r_D3725 = 1;
        while (r_D3725 <= n)
        {
            summ = \
summ+u(r_D3725)*exp(2.*3.14159265358979323846*lm_complex(0,1)
                *(b*1.)*(r_D3725+-1.)*((s_D3728+-1.)/(n*1.)));
            r_D3725 = r_D3725+1;
        }
        v(s_D3728) = summ;
        s_D3728 = s_D3728+1;
    }
    return v*(1./pow(n*1., (1.+-af)/2.));
}

lm_complexN system_TInverseFourier ( const lm_complexN &bff)
{
    return system_TInverseFourier3 (bff, 0, 1);
}

lm_complexN system_TInverseFourier3 ( const lm_complexN &v, const int &af, 
    const int &b)
{
    int n;
    lm_complex summ = 0;
    lm_complexN u(0);
    u = v;
    int r;
    int s;
    int s_D3866;
    int r_D3869;

    n = v.dimension(1);
    r_D3869 = 1;
    while (r_D3869 <= n)
    {
        summ = 0;
        s_D3866 = 1;
        while (s_D3866 <= n)
        {
            summ = summ+v(s_D3866)*exp(-2.*3.14159265358979323846*
                lm_complex(0,1)*(b*1.)*(r_D3869+-1.)*((s_D3866+-1.)/(n*1.)));
            s_D3866 = s_D3866+1;
        }
        u(r_D3869) = summ;
        r_D3869 = r_D3869+1;
    }
    return u*(1./pow(n*1., (1.+af)/2.));
}

intN system_TRotateLeft ( const intN &ar)
{
    int m = ar.dimension(1);
    intN res(m);

    res.Set(R(1,m+-1),ar(R(2,m)));
    res(m) = ar(1);
    return res;
}

intN system_TRotateRight ( const intN &ar)
{
    int m = ar.dimension(1);
    intN res(m);

    res.Set(R(2,m),ar(R(1,m+-1)));
    res(1) = ar(m);
    return res;
}

void system_TsystemInit ()
{
; 
}

double system_TErfOne ( const double &x)
{
    double mc_T1;
    double mc_T2;
    double mc_T3;
    double mc_T4;
    double mc_T5;
    double mc_T6;
    double mc_T7;
    double mc_T8;
    double mc_T9;
    double mc_T10;
    double mc_T11;
    double mc_T12;
    double mc_T13;
    double mc_T14;
    double mc_T15;
    double mc_T16;
    double mc_T17;
    double mc_T18;
    double mc_T19;
    double mc_T20;
    double mc_T21;
    double mc_T22;
    double mc_T23;
    double mc_T24;
    double mc_T25;
    double mc_T26;
    double mc_T27;
    double mc_T28;
    double mc_T29;
    double mc_T30;
    double mc_T31;
    double mc_T32;
    double mc_T33;
    double mc_T34;
    double mc_T35;
    double mc_T36;
    double mc_T37;
    double mc_T38;
    double mc_T39;
    double mc_T40;
    double mc_T41;
    double mc_T42;
    double mc_T43;
    double mc_T44;
    double mc_T45;
    double mc_T46;
    double mc_T47;
    double mc_T48;
    double mc_T49;
    double mc_T50;
    double mc_T51;
    double mc_T52;
    double mc_T53;
    double mc_T54;
    double mc_T55;
    double mc_T56;
    double mc_T57;
    double mc_T58;
    double mc_T59;
    double mc_T60;
    double mc_T61;
    double mc_T62;
    double mc_T63;
    double mc_T64;
    mc_T1 = (x*x);
    mc_T2 = 40.*mc_T1;
    mc_T3 = 39.+mc_T2;
    mc_T4 = -38.*mc_T1;
    mc_T5 = mc_T4/mc_T3;
    mc_T6 = 37.+mc_T5;
    mc_T7 = 36.*mc_T1;
    mc_T8 = mc_T7/mc_T6;
    mc_T9 = 35.+mc_T8;
    mc_T10 = -34.*mc_T1;
    mc_T11 = mc_T10/mc_T9;
    mc_T12 = 33.+mc_T11;
    mc_T13 = 32.*mc_T1;
    mc_T14 = mc_T13/mc_T12;
    mc_T15 = 31.+mc_T14;
    mc_T16 = -30.*mc_T1;
    mc_T17 = mc_T16/mc_T15;
    mc_T18 = 29.+mc_T17;
    mc_T19 = 28.*mc_T1;
    mc_T20 = mc_T19/mc_T18;
    mc_T21 = 27.+mc_T20;
    mc_T22 = -26.*mc_T1;
    mc_T23 = mc_T22/mc_T21;
    mc_T24 = 25.+mc_T23;
    mc_T25 = 24.*mc_T1;
    mc_T26 = mc_T25/mc_T24;
    mc_T27 = 23.+mc_T26;
    mc_T28 = -22.*mc_T1;
    mc_T29 = mc_T28/mc_T27;
    mc_T30 = 21.+mc_T29;
    mc_T31 = 20.*mc_T1;
    mc_T32 = mc_T31/mc_T30;
    mc_T33 = 19.+mc_T32;
    mc_T34 = -18.*mc_T1;
    mc_T35 = mc_T34/mc_T33;
    mc_T36 = 17.+mc_T35;
    mc_T37 = 16.*mc_T1;
    mc_T38 = mc_T37/mc_T36;
    mc_T39 = 15.+mc_T38;
    mc_T40 = -14.*mc_T1;
    mc_T41 = mc_T40/mc_T39;
    mc_T42 = 13.+mc_T41;
    mc_T43 = 12.*mc_T1;
    mc_T44 = mc_T43/mc_T42;
    mc_T45 = 11.+mc_T44;
    mc_T46 = -10.*mc_T1;
    mc_T47 = mc_T46/mc_T45;
    mc_T48 = 9.+mc_T47;
    mc_T49 = 8.*mc_T1;
    mc_T50 = mc_T49/mc_T48;
    mc_T51 = 7.+mc_T50;
    mc_T52 = -6.*mc_T1;
    mc_T53 = mc_T52/mc_T51;
    mc_T54 = 5.+mc_T53;
    mc_T55 = 4.*mc_T1;
    mc_T56 = mc_T55/mc_T54;
    mc_T57 = 3.+mc_T56;
    mc_T58 = -2.*mc_T1;
    mc_T59 = mc_T58/mc_T57;
    mc_T60 = 1.+mc_T59;
    mc_T61 = -1.*mc_T1;
    mc_T62 = pow(2.718281828459045, mc_T61);
    mc_T63 = 1.1283791670955126*mc_T62*x;
    mc_T64 = mc_T63/mc_T60;
    return mc_T64;
}

double system_TErfTwo ( const double &x)
{
    double mc_T1;
    double mc_T2;
    double mc_T3;
    double mc_T4;
    double mc_T5;
    double mc_T6;
    double mc_T7;
    double mc_T8;
    double mc_T9;
    double mc_T10;
    double mc_T11;
    double mc_T12;
    double mc_T13;
    double mc_T14;
    double mc_T15;
    double mc_T16;
    double mc_T17;
    double mc_T18;
    double mc_T19;
    double mc_T20;
    double mc_T21;
    double mc_T22;
    double mc_T23;
    double mc_T24;
    double mc_T25;
    double mc_T26;
    double mc_T27;
    double mc_T28;
    double mc_T29;
    double mc_T30;
    double mc_T31;
    double mc_T32;
    double mc_T33;
    double mc_T34;
    double mc_T35;
    double mc_T36;
    double mc_T37;
    double mc_T38;
    double mc_T39;
    double mc_T40;
    double mc_T41;
    double mc_T42;
    double mc_T43;
    double mc_T44;
    double mc_T45;
    double mc_T46;
    mc_T1 = 1.4142135623730951*x;
    mc_T2 = 20.+mc_T1;
    mc_T3 = 19./mc_T2;
    mc_T4 = mc_T1+mc_T3;
    mc_T5 = 18./mc_T4;
    mc_T6 = mc_T1+mc_T5;
    mc_T7 = 17./mc_T6;
    mc_T8 = mc_T1+mc_T7;
    mc_T9 = 16./mc_T8;
    mc_T10 = mc_T1+mc_T9;
    mc_T11 = 15./mc_T10;
    mc_T12 = mc_T1+mc_T11;
    mc_T13 = 14./mc_T12;
    mc_T14 = mc_T1+mc_T13;
    mc_T15 = 13./mc_T14;
    mc_T16 = mc_T1+mc_T15;
    mc_T17 = 12./mc_T16;
    mc_T18 = mc_T1+mc_T17;
    mc_T19 = 11./mc_T18;
    mc_T20 = mc_T1+mc_T19;
    mc_T21 = 10./mc_T20;
    mc_T22 = mc_T1+mc_T21;
    mc_T23 = 9./mc_T22;
    mc_T24 = mc_T1+mc_T23;
    mc_T25 = 8./mc_T24;
    mc_T26 = mc_T1+mc_T25;
    mc_T27 = 7./mc_T26;
    mc_T28 = mc_T1+mc_T27;
    mc_T29 = 6./mc_T28;
    mc_T30 = mc_T1+mc_T29;
    mc_T31 = 5./mc_T30;
    mc_T32 = mc_T1+mc_T31;
    mc_T33 = 4./mc_T32;
    mc_T34 = mc_T1+mc_T33;
    mc_T35 = 3./mc_T34;
    mc_T36 = mc_T1+mc_T35;
    mc_T37 = 2./mc_T36;
    mc_T38 = mc_T1+mc_T37;
    mc_T39 = 1/mc_T38;
    mc_T40 = mc_T1+mc_T39;
    mc_T41 = (x*x);
    mc_T42 = -1.*mc_T41;
    mc_T43 = pow(2.718281828459045, mc_T42);
    mc_T44 = -0.7978845608028654*mc_T43;
    mc_T45 = mc_T44/mc_T40;
    mc_T46 = 1.+mc_T45;
    return mc_T46;
}
\
\>", "Print",
 CellChangeTimes->{{3.50918410471894*^9, 3.50918411296894*^9}, {
   3.509184303672065*^9, 3.50918431193769*^9}, 3.50918441771894*^9, 
   3.509184573922065*^9, 3.50918462240644*^9, 3.509184738515815*^9, 
   3.509184777953315*^9, {3.509185015890815*^9, 3.509185024328315*^9}, {
   3.50918506796894*^9, 3.50918507606269*^9}, 3.50918515984394*^9, 
   3.509185320703315*^9, 3.509185370578315*^9, 3.509185413547065*^9, 
   3.50918547331269*^9, {3.509185534922065*^9, 3.509185571140815*^9}, {
   3.50918614331269*^9, 3.509186151390815*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["!!system.mh", "Input",
 InitializationCell->True],

Cell["\<\
(* File automatically generated by MathCode. *)

BeginPackage[\"system`\", {MathCodeContexts}]

(* These were the compilation options when this file was created. *)
SetCompilationOptions[CallBackFunctions -> {}, Compiler -> \"\", \
CompilerOptions -> \"\", DebugFlag -> False, DisabledMathLinkFunctions -> {}, \
DownValueEvaluator -> 0, EnabledMathLinkFunctions -> {}, EvaluateFunctions -> \
{ErfOne, ErfTwo}, InlineFlag -> False, InterpolatingFunctionNames -> {}, \
Language -> \"C++\", LinkerOptions -> \"\", MacroRules -> {}, \
MainFileAndFunction -> \"int main(){return 0;}\", MathCodeMakeFile -> \
\"C:\\\\MathCode\\\\mk4/System/compwin.mak\", MCIConverter -> True, \
RangeCheckFlag -> True, StandAloneExecutable -> False, UnCompiledFunctions -> \
{}, NeedsExternalObjectModule -> \
{\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\inverse\"}, \
NeedsExternalLibrary -> {\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\
\\\\Blas_IFORT91\", \
\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\lapack_IFORT91\", \
\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\ifconsol\", \
\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\libifcoremt\", \"C:\\\
\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\libifport\", \
\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\libirc\", \
\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\libmmt\"}]

Erf
ErfOne
ErfTwo
Fourier
Fourier3
Inverse
InverseFourier
InverseFourier3
RotateLeft
RotateRight
systemInit

Begin[\"`Private`\"]
({Real} -> Real) Erf[system`Private`x_] := ExternalFunction[]

({Real} -> Real) ErfOne[system`Private`x_] := ExternalFunction[]

({Real} -> Real) ErfTwo[system`Private`x_] := ExternalFunction[]

({ArrayOf[Complex, {_}]} -> ArrayOf[Complex, {_}]) Fourier[aff_] := \
ExternalFunction[]

({ArrayOf[Complex, {_}], Integer, Integer} -> ArrayOf[Complex, {_}]) \
Fourier3[system`Private`u_, af_, b_] := ExternalFunction[]

({ArrayOf[Real, {_, _}]} -> ArrayOf[Real, {_, _}]) Inverse[aff_] := \
ExternalFunction[aff, ExternalPrefix -> \"System_T\"]

({ArrayOf[Complex, {_}]} -> ArrayOf[Complex, {_}]) InverseFourier[bff_] := \
ExternalFunction[]

({ArrayOf[Complex, {_}], Integer, Integer} -> ArrayOf[Complex, {_}]) \
InverseFourier3[v_, af_, b_] := ExternalFunction[]

({ArrayOf[Integer, {_}]} -> ArrayOf[Integer, {_}]) RotateLeft[ar_] := \
ExternalFunction[]

({ArrayOf[Integer, {_}]} -> ArrayOf[Integer, {_}]) RotateRight[ar_] := \
ExternalFunction[]

({} -> Null) systemInit[] := ExternalProcedure[]


End[]
EndPackage[]\
\>", "Print",
 CellChangeTimes->{{3.50918387365644*^9, 3.509183880515815*^9}, {
   3.50918410478144*^9, 3.50918411300019*^9}, {3.50918430371894*^9, 
   3.509184311984565*^9}, 3.509184417765815*^9, 3.50918457396894*^9, {
   3.509184622453315*^9, 3.50918463931269*^9}, 3.50918473859394*^9, 
   3.50918477803144*^9, {3.50918501596894*^9, 3.50918502440644*^9}, {
   3.50918506803144*^9, 3.509185076109565*^9}, 3.509185159890815*^9, 
   3.50918532075019*^9, 3.50918537062519*^9, 3.50918541359394*^9, 
   3.509185473359565*^9, {3.50918553496894*^9, 3.50918557118769*^9}, {
   3.509186143359565*^9, 3.50918615143769*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Compile the C++ files to binaries and Possibly link into an executable\
\>", "Subsubsection",
 InitializationCell->True],

Cell["MakeBinary[]", "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Testing compiled code and cleanup", "Section",
 InitializationCell->True],

Cell[TextData[{
 "A prerequisite for testing the compiled code via a ",
 StyleBox["MathLink",
  FontSlant->"Italic"],
 " connection as below, is that GenerateMainFileAndFunction is called within  \
the CompilationScript to produce a main file and function, which is needed  \
for producing the executable to be produced via ",
 StyleBox["MathLink",
  FontSlant->"Italic"],
 "."
}], "Text",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["Install", "Subsection",
 InitializationCell->True],

Cell["Installs the external process into Mathematica", "Text",
 InitializationCell->True],

Cell["\<\
Remove[RotLeft];
Remove[RotRight];\
\>", "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"installedCodeHandle", "=", 
  RowBox[{"InstallCode", "[", "\"\<system\>\"", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"system\"\>", "\[InvisibleSpace]", "\<\" is installed.\"\>"}],
  SequenceForm["system", " is installed."],
  Editable->False]], "Print",
 CellChangeTimes->{{3.50918410887519*^9, 3.50918411675019*^9}, {
   3.509184307797065*^9, 3.50918431571894*^9}, 3.509184421703315*^9, 
   3.509184578047065*^9, 3.509184626297065*^9, 3.509184742672065*^9, 
   3.50918478187519*^9, {3.509185019922065*^9, 3.50918502815644*^9}, {
   3.50918507184394*^9, 3.509185079828315*^9}, 3.50918516378144*^9, 
   3.509185324890815*^9, 3.50918537446894*^9, 3.509185417422065*^9, 
   3.509185477234565*^9, {3.50918553881269*^9, 3.509185574922065*^9}, {
   3.509186147265815*^9, 3.509186155140815*^9}}],

Cell[BoxData[
 RowBox[{"LinkObject", "[", 
  RowBox[{"\<\"\\\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\\
systemml.exe\\\"\"\>", ",", "23", ",", "8"}], "]"}]], "Output",
 CellChangeTimes->{{3.509183878797065*^9, 3.509183884672065*^9}, {
   3.50918410887519*^9, 3.509184116765815*^9}, {3.509184307797065*^9, 
   3.50918431571894*^9}, 3.509184421703315*^9, 3.509184578047065*^9, 
   3.509184626297065*^9, 3.509184742672065*^9, 3.50918478187519*^9, {
   3.509185019922065*^9, 3.50918502815644*^9}, {3.50918507184394*^9, 
   3.509185079828315*^9}, 3.50918516378144*^9, 3.509185324890815*^9, 
   3.50918537446894*^9, 3.509185417422065*^9, 3.509185477234565*^9, {
   3.50918553881269*^9, 3.509185574922065*^9}, {3.509186147265815*^9, 
   3.509186155140815*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Test cases", "Subsection",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["RotateLeft[{1,2,3,4}]=={2,3,4,1}", "Input",
 InitializationCell->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{{3.50918387887519*^9, 3.509183884703315*^9}, {
   3.509184108922065*^9, 3.509184116765815*^9}, {3.509184307828315*^9, 
   3.509184315734565*^9}, 3.50918442175019*^9, 3.509184578078315*^9, 
   3.509184626328315*^9, 3.50918474268769*^9, 3.50918478190644*^9, {
   3.50918501993769*^9, 3.50918502818769*^9}, {3.509185071890815*^9, 
   3.50918507984394*^9}, 3.50918516381269*^9, 3.50918532493769*^9, 
   3.509185374515815*^9, 3.50918541746894*^9, 3.50918547728144*^9, {
   3.509185538859565*^9, 3.50918557493769*^9}, {3.50918614731269*^9, 
   3.50918615515644*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["RotateRight[{1,2,3,4}]=={4,1,2,3}", "Input",
 InitializationCell->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{{3.509183878890815*^9, 3.50918388471894*^9}, {
   3.50918410893769*^9, 3.50918411678144*^9}, {3.50918430784394*^9, 
   3.509184315734565*^9}, 3.509184421765815*^9, 3.50918457809394*^9, 
   3.50918462634394*^9, 3.50918474271894*^9, 3.50918478193769*^9, {
   3.50918501996894*^9, 3.509185028203315*^9}, {3.50918507190644*^9, 
   3.509185079859565*^9}, 3.509185163828315*^9, 3.509185324953315*^9, 
   3.50918537453144*^9, 3.509185417484565*^9, 3.509185477297065*^9, {
   3.50918553887519*^9, 3.50918557493769*^9}, {3.509186147328315*^9, 
   3.509186155172065*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Erf", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "4"}], ",", "4"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwt2nc81d//AHDrXndYaX+iZBWJEonSORUlokJmSKG9aNu7IasSkuyGmYbt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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.509184108984565*^9, 3.509184116828315*^9}, {
   3.50918430790644*^9, 3.50918431578144*^9}, 3.50918442181269*^9, 
   3.50918457815644*^9, 3.509184626390815*^9, 3.50918474278144*^9, 
   3.50918478200019*^9, {3.50918502003144*^9, 3.509185028265815*^9}, {
   3.50918507196894*^9, 3.50918507990644*^9}, 3.509185163890815*^9, 
   3.50918532500019*^9, 3.509185374578315*^9, 3.509185417547065*^9, 
   3.50918547734394*^9, {3.509185538922065*^9, 3.50918557500019*^9}, {
   3.50918614737519*^9, 3.50918615521894*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Erf", "[", "x", "]"}], "-", 
    RowBox[{"System`Erf", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "4"}], ",", "4"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxF22k4Vd/bB/BjSEioTEUlIYkyFGIvJCIhZArJVEkSEkJIZSgZI0lmEUKl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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.50918410906269*^9, 3.50918411687519*^9}, {
   3.509184307953315*^9, 3.50918431584394*^9}, 3.509184421859565*^9, 
   3.509184578203315*^9, 3.509184626453315*^9, 3.509184742828315*^9, 
   3.509184782047065*^9, {3.509185020078315*^9, 3.50918502831269*^9}, {
   3.509185072015815*^9, 3.509185079953315*^9}, 3.50918516393769*^9, 
   3.509185325047065*^9, 3.509185374640815*^9, 3.50918541759394*^9, 
   3.50918547740644*^9, {3.509185538984565*^9, 3.509185575047065*^9}, {
   3.50918614743769*^9, 3.509186155265815*^9}}]
}, Open  ]],

Cell["Fourier functions", "Text",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"u", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Sin", "[", "axx", "]"}], "+", 
       RowBox[{"\[ImaginaryI]", " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"Sin", "[", "bxx", "]"}], "]"}]}]}], ",", 
      RowBox[{"{", 
       RowBox[{"axx", ",", "1", ",", "4"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"bxx", ",", "1", ",", "4"}], "}"}]}], "]"}], "//", "Flatten"}],
    "//", "N"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"0.8414709848078965`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.7456241416655579`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.8414709848078965`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.7890723435728884`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.8414709848078965`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.14065207678644337`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.8414709848078965`", "\[InvisibleSpace]", "-", 
    RowBox[{"0.6866002607386249`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.9092974268256817`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.7456241416655579`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.9092974268256817`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.7890723435728884`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.9092974268256817`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.14065207678644337`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.9092974268256817`", "\[InvisibleSpace]", "-", 
    RowBox[{"0.6866002607386249`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.1411200080598672`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.7456241416655579`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.1411200080598672`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.7890723435728884`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.1411200080598672`", "\[InvisibleSpace]", "+", 
    RowBox[{"0.14065207678644337`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{"0.1411200080598672`", "\[InvisibleSpace]", "-", 
    RowBox[{"0.6866002607386249`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "0.7568024953079282`"}], "+", 
    RowBox[{"0.7456241416655579`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "0.7568024953079282`"}], "+", 
    RowBox[{"0.7890723435728884`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "0.7568024953079282`"}], "+", 
    RowBox[{"0.14065207678644337`", " ", "\[ImaginaryI]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "0.7568024953079282`"}], "-", 
    RowBox[{"0.6866002607386249`", " ", "\[ImaginaryI]"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.509183879047065*^9, 3.50918388484394*^9}, {
   3.509184109078315*^9, 3.509184116890815*^9}, {3.50918430796894*^9, 
   3.50918431584394*^9}, 3.50918442187519*^9, 3.50918457821894*^9, 
   3.509184626453315*^9, 3.50918474284394*^9, 3.50918478206269*^9, {
   3.50918502009394*^9, 3.509185028328315*^9}, {3.50918507203144*^9, 
   3.50918507996894*^9}, 3.509185163953315*^9, 3.50918532506269*^9, 
   3.509185374640815*^9, 3.509185417609565*^9, 3.50918547740644*^9, {
   3.509185538984565*^9, 3.50918557506269*^9}, {3.50918614743769*^9, 
   3.50918615528144*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Fourier3", "[", 
    RowBox[{"u", ",", "0", ",", "1"}], "]"}], "-", " ", 
   RowBox[{"System`Fourier", "[", 
    RowBox[{"u", ",", 
     RowBox[{
      StyleBox["FourierParameters", "MR"], " ", 
      StyleBox["->", "MR"], 
      StyleBox[
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], "MR"]}]}], "]"}]}], "//", 
  "Chop"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.509183879078315*^9, 3.509183884859565*^9}, {
   3.50918410909394*^9, 3.509184116890815*^9}, {3.509184307984565*^9, 
   3.509184315859565*^9}, 3.509184421890815*^9, 3.509184578234565*^9, 
   3.50918462646894*^9, 3.50918474287519*^9, 3.50918478209394*^9, {
   3.50918502012519*^9, 3.509185028359565*^9}, {3.509185072047065*^9, 
   3.509185079984565*^9}, 3.50918516396894*^9, 3.509185325078315*^9, 
   3.509185374672065*^9, 3.50918541762519*^9, 3.50918547743769*^9, {
   3.509185539015815*^9, 3.509185575078315*^9}, {3.509186147453315*^9, 
   3.509186155297065*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InverseFourier3", "[", 
    RowBox[{"u", ",", "0", ",", "1"}], "]"}], " ", "-", " ", 
   RowBox[{"System`InverseFourier", "[", 
    RowBox[{"u", ",", 
     RowBox[{
      StyleBox["FourierParameters", "MR"], " ", 
      StyleBox["->", "MR"], 
      StyleBox[
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], "MR"]}]}], "]"}]}], "//", 
  "Chop"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.50918387909394*^9, 3.50918388487519*^9}, {
   3.509184109109565*^9, 3.50918411690644*^9}, {3.50918430800019*^9, 
   3.50918431587519*^9}, 3.50918442190644*^9, 3.509184578234565*^9, 
   3.509184626484565*^9, 3.509184742890815*^9, 3.509184782109565*^9, {
   3.509185020140815*^9, 3.50918502837519*^9}, {3.50918507206269*^9, 
   3.50918508000019*^9}, 3.509185163984565*^9, 3.50918532509394*^9, 
   3.50918537468769*^9, 3.509185417640815*^9, 3.509185477453315*^9, {
   3.50918553903144*^9, 3.50918557509394*^9}, {3.50918614746894*^9, 
   3.509186155297065*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["UnInstall and cleanup", "Section",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["UninstallCode[\"system\"]", "Input",
 InitializationCell->True],

Cell[BoxData["\<\"\\\"C:\\\\MathCode\\\\mk4\\\\lib\\\\stdpackages\\\\src\\\\\
systemml.exe\\\"\"\>"], "Output",
 CellChangeTimes->{{3.509184109109565*^9, 3.509184116922065*^9}, {
   3.50918430803144*^9, 3.50918431590644*^9}, 3.509184421922065*^9, 
   3.50918457825019*^9, 3.50918462650019*^9, 3.509184742922065*^9, 
   3.50918478215644*^9, {3.509185020203315*^9, 3.50918502840644*^9}, {
   3.50918507209394*^9, 3.509185080015815*^9}, 3.50918516400019*^9, 
   3.50918532512519*^9, 3.509185374703315*^9, 3.509185417672065*^9, 
   3.50918547746894*^9, {3.509185539047065*^9, 3.50918557512519*^9}, {
   3.509186147484565*^9, 3.50918615531269*^9}}]
}, Open  ]],

Cell["\<\
CleanMathCodeFiles[CleanAllBut->{\".obj\",\".o\",\".cc\",\".mh\"},Confirm->\
False]\
\>", "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fix System.mh to avoid localized paths", "Subtitle",
 InitializationCell->True],

Cell[TextData[{
 "When system.mh is created, it contains the paths valid for local computer \
and it makes no sense when this file is distributed to other computers. This \
script uses ",
 StyleBox["$MCRoot ",
  FontWeight->"Bold"],
 "to evaluate actual paths.\n In case of MINGW32 the $MCRoot appears with \
duplicated \\\\ in system.mh.\n In case of CYGWIN the  $MCRoot  appears with  \
/cygdrive/c/... syntax, with duplicated \\\\ ."
}], "Text",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"s", "=", 
   RowBox[{"Import", "[", 
    RowBox[{"\"\<system.mh\>\"", ",", "\"\<Lines\>\""}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"s", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"StringReplace", "[", " ", 
        RowBox[{"#", ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"\"\<\\\"\>\"", "<>", "$MCRoot"}], "->", 
            "\"\<$MCRoot<>\\\"\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"\"\<\\\"\>\"", "<>", 
             RowBox[{"Disk2CygwinStyle", "[", 
              RowBox[{"StringReplace", "[", 
               RowBox[{"$MCRoot", ",", 
                RowBox[{"\"\<\\\\\>\"", "->", "\"\<\\\\\\\\\>\""}]}], "]"}], 
              "]"}]}], "->", "\"\<$MCRoot<>\\\"\>\""}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"\"\<\\\"\>\"", "<>", 
             RowBox[{"StringReplace", "[", 
              RowBox[{"$MCRoot", ",", 
               RowBox[{"\"\<\\\\\>\"", "->", "\"\<\\\\\\\\\>\""}]}], "]"}]}], 
            "->", "\"\<$MCRoot<>\\\"\>\""}]}], "}"}]}], "\[IndentingNewLine]",
         "]"}], "<>", "\"\<\\n\>\""}], "&"}], ",", "s"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Export", "[", 
    RowBox[{"\"\<system.mh\>\"", ",", "s", ",", "\"\<Lines\>\""}], "]"}], 
   ";"}], "\[IndentingNewLine]"}]], "Input",
 InitializationCell->True]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowToolbars->"EditBar",
WindowSize->{1272, 921},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
Magnification->1,
FrontEndVersion->"7.0 for Microsoft Windows (32-bit) (November 10, 2008)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 170, 6, 71, "Section",
 InitializationCell->True],
Cell[740, 30, 251, 6, 83, "Text",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1028, 41, 73, 1, 71, "Section",
 InitializationCell->True],
Cell[1104, 44, 101, 2, 31, "Input",
 InitializationCell->True],
Cell[1208, 48, 171, 5, 31, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1416, 58, 122, 6, 71, "Section",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1563, 68, 57, 1, 36, "Subsection",
 InitializationCell->True],
Cell[1623, 71, 99, 1, 29, "Text",
 InitializationCell->True],
Cell[1725, 74, 431, 12, 72, "Input",
 InitializationCell->True],
Cell[2159, 88, 494, 15, 31, "Input",
 InitializationCell->True],
Cell[2656, 105, 57, 1, 29, "Text",
 InitializationCell->True],
Cell[2716, 108, 188, 5, 31, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[2941, 118, 79, 1, 36, "Subsection",
 InitializationCell->True],
Cell[3023, 121, 64, 1, 31, "Input",
 InitializationCell->True],
Cell[3090, 124, 113, 3, 29, "Text",
 InitializationCell->True],
Cell[3206, 129, 134, 4, 47, "Text",
 InitializationCell->True],
Cell[3343, 135, 62, 1, 31, "Input",
 InitializationCell->True],
Cell[3408, 138, 249, 14, 218, "Input",
 InitializationCell->True],
Cell[3660, 154, 61, 1, 31, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[3746, 159, 70, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[3819, 162, 51, 1, 31, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3919, 169, 79, 1, 36, "Subsection",
 InitializationCell->True],
Cell[4001, 172, 145, 4, 47, "Text",
 InitializationCell->True],
Cell[4149, 178, 66, 1, 31, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[4240, 183, 78, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[4321, 186, 51, 1, 31, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[4421, 193, 79, 1, 36, "Subsection",
 InitializationCell->True],
Cell[4503, 196, 66, 1, 31, "Input",
 InitializationCell->True],
Cell[4572, 199, 96, 1, 29, "Text",
 InitializationCell->True],
Cell[4671, 202, 607, 17, 31, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[5303, 223, 63, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[5369, 226, 237, 11, 167, "Input",
 InitializationCell->True],
Cell[5609, 239, 60, 1, 29, "Text",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[5694, 244, 64, 1, 31, "Input",
 InitializationCell->True],
Cell[5761, 247, 667, 11, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6477, 264, 62, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[6542, 267, 236, 11, 167, "Input",
 InitializationCell->True],
Cell[6781, 280, 62, 1, 29, "Text",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[6868, 285, 76, 1, 31, "Input",
 InitializationCell->True],
Cell[6947, 288, 618, 9, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[7614, 303, 59, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[7676, 306, 1201, 41, 65, "Text",
 InitializationCell->True],
Cell[8880, 349, 576, 15, 65, "Text",
 InitializationCell->True],
Cell[9459, 366, 727, 20, 52, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[10223, 391, 55, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[10281, 394, 829, 27, 47, "Input",
 InitializationCell->True],
Cell[11113, 423, 596, 22, 63, "Input",
 InitializationCell->True],
Cell[11712, 447, 718, 23, 47, "Input",
 InitializationCell->True],
Cell[12433, 472, 565, 20, 56, "Input",
 InitializationCell->True],
Cell[13001, 494, 655, 18, 112, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[13693, 517, 69, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[13765, 520, 2501, 70, 152, "Input",
 InitializationCell->True],
Cell[16269, 592, 2529, 71, 152, "Input",
 InitializationCell->True],
Cell[18801, 665, 495, 14, 92, "Input",
 InitializationCell->True],
Cell[19299, 681, 509, 14, 92, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[19845, 700, 74, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[19944, 705, 49, 1, 31, "Input",
 InitializationCell->True],
Cell[19996, 708, 633, 9, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20690, 724, 63, 1, 36, "Subsection",
 InitializationCell->True],
Cell[20756, 727, 56, 1, 31, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20861, 734, 80, 1, 71, "Section",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[20966, 739, 69, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[21038, 742, 218, 5, 65, "Text",
 InitializationCell->True],
Cell[21259, 749, 1616, 40, 252, "Input",
 InitializationCell->True],
Cell[22878, 791, 2019, 50, 332, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[24922, 845, 124, 3, 31, "Input"],
Cell[25049, 850, 441, 6, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[25527, 861, 2668, 65, 432, "Input",
 InitializationCell->True],
Cell[28198, 928, 635, 8, 23, "Print"]
}, Open  ]],
Cell[28848, 939, 1575, 40, 252, "Input",
 InitializationCell->True],
Cell[30426, 981, 816, 22, 132, "Input",
 InitializationCell->True],
Cell[31245, 1005, 669, 18, 52, "Input",
 InitializationCell->True],
Cell[31917, 1025, 1667, 43, 212, "Input",
 InitializationCell->True],
Cell[33587, 1070, 990, 26, 212, "Input",
 Evaluatable->False,
 InitializationCell->True],
Cell[34580, 1098, 1440, 35, 252, "Input",
 InitializationCell->True],
Cell[36023, 1135, 412, 11, 72, "Input",
 InitializationCell->True],
Cell[36438, 1148, 407, 11, 72, "Input",
 InitializationCell->True],
Cell[36848, 1161, 414, 11, 72, "Input",
 InitializationCell->True],
Cell[37265, 1174, 412, 11, 72, "Input",
 InitializationCell->True],
Cell[37680, 1187, 415, 11, 72, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[38132, 1203, 74, 1, 27, "Subsubsection",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[38231, 1208, 55, 1, 31, "Input",
 InitializationCell->True],
Cell[38289, 1211, 7820, 348, 5399, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46146, 1564, 55, 1, 31, "Input",
 InitializationCell->True],
Cell[46204, 1567, 3157, 79, 727, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[49410, 1652, 130, 3, 27, "Subsubsection",
 InitializationCell->True],
Cell[49543, 1657, 56, 1, 31, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[49648, 1664, 79, 1, 71, "Section",
 InitializationCell->True],
Cell[49730, 1667, 416, 11, 47, "Text",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[50171, 1682, 56, 1, 36, "Subsection",
 InitializationCell->True],
Cell[50230, 1685, 89, 1, 29, "Text",
 InitializationCell->True],
Cell[50322, 1688, 86, 4, 48, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[50433, 1696, 145, 3, 31, "Input",
 InitializationCell->True],
Cell[50581, 1701, 719, 12, 23, "Print"],
Cell[51303, 1715, 767, 12, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[52119, 1733, 59, 1, 36, "Subsection",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[52203, 1738, 76, 1, 31, "Input",
 InitializationCell->True],
Cell[52282, 1741, 611, 9, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52930, 1755, 77, 1, 31, "Input",
 InitializationCell->True],
Cell[53010, 1758, 612, 9, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53659, 1772, 268, 8, 31, "Input",
 InitializationCell->True],
Cell[53930, 1782, 15850, 264, 238, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[69817, 2051, 332, 10, 31, "Input",
 InitializationCell->True],
Cell[70152, 2063, 12285, 205, 238, "Output"]
}, Open  ]],
Cell[82452, 2271, 60, 1, 29, "Text",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[82537, 2276, 518, 16, 31, "Input",
 InitializationCell->True],
Cell[83058, 2294, 2683, 48, 69, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85778, 2347, 423, 14, 31, "Input",
 InitializationCell->True],
Cell[86204, 2363, 810, 14, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87051, 2382, 442, 14, 31, "Input",
 InitializationCell->True],
Cell[87496, 2398, 804, 14, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[88361, 2419, 67, 1, 71, "Section",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[88453, 2424, 69, 1, 31, "Input",
 InitializationCell->True],
Cell[88525, 2427, 643, 9, 30, "Output"]
}, Open  ]],
Cell[89183, 2439, 135, 4, 31, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[89355, 2448, 85, 1, 49, "Subtitle",
 InitializationCell->True],
Cell[89443, 2451, 475, 10, 65, "Text",
 InitializationCell->True],
Cell[89921, 2463, 184, 6, 31, "Input",
 InitializationCell->True],
Cell[90108, 2471, 1183, 30, 112, "Input",
 InitializationCell->True],
Cell[91294, 2503, 208, 6, 52, "Input",
 InitializationCell->True]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
